#!/usr/bin/env python
# encoding: utf-8
r"""
Module containing all Pyclaw solution objects
"""

class State(object):
    r"""
    A PyClaw State object contains the current state on a particular patch,
    including the unknowns q, the time t, and the auxiliary coefficients aux.

    The variables num_eqn and num_aux determine the length of the first
    dimension of the q and aux arrays.

    :State Data:
    
        The arrays :attr:`q`, and :attr:`aux` have variable 
        extents based on the patch dimensions and the values of 
        :attr:`num_eqn` and :attr:`num_aux`.  
 
    A State object is automatically created upon instantiation of a Solution object
    from a Domain object:

        >>> from clawpack import pyclaw
        >>> x = pyclaw.Dimension('x',0.0,1.0,100)
        >>> domain = pyclaw.Domain(x)
        >>> num_eqn = 1
        >>> solution = pyclaw.Solution(num_eqn,domain)
        >>> print solution.state
        PyClaw State object
        Patch dimensions: [100]
        Time  t=0.0
        Number of conserved quantities: 1
        <BLANKLINE>

    A State lives on a Patch, and can be instantiated directly
    by first creating a Patch:

        >>> x = pyclaw.Dimension('x',0.,1.,100)
        >>> patch = pyclaw.Patch((x))

    The arguments to the constructor are the patch, the number of equations,
    and the number of auxiliary fields:

        >>> state = pyclaw.State(patch,3,2)
        >>> state.q.shape
        (3, 100)
        >>> state.aux.shape
        (2, 100)
        >>> state.t
        0.0

    Note that state.q and state.aux are initialized as empty arrays (not zeroed).
    Additional parameters, such as scalar values that are used in the Riemann solver,
    can be set using the dictionary state.problem_data.
    """

    def __getattr__(self, key):
        if key in ('num_dim', 'p_centers', 'p_edges', 'c_centers', 'c_edges',
                   'num_cells', 'lower', 'upper', 'delta', 'centers', 'edges',
                   'gauges'):
            return self._get_grid_attribute(key)
        else:
            raise AttributeError("'State' object has no attribute '"+key+"'")

    def _get_grid_attribute(self, name):
        r"""
        Return grid attribute
        
        :Output:
         - (id) - Value of attribute from ``grid``
        """
        return getattr(self.grid,name)
 

    # ========== Property Definitions ========================================
    @property
    def num_eqn(self):
        r"""(int) - Number of unknowns (components of q)"""
        if self.q is None:
            raise Exception('state.num_eqn has not been set.')
        else: return self.q.shape[0]

    @property
    def num_aux(self):
        r"""(int) - Number of auxiliary fields"""
        if self.aux is not None: return self.aux.shape[0]
        else: return 0

    @property
    def grid(self):
        return self.patch.grid

    @property
    def mp(self):
        r"""(int) - Number of derived quantities"""
        if self.p is not None: return self.p.shape[0]
        else: return 0
    @mp.setter
    def mp(self,mp):
        if self.p is not None:
            raise Exception('Cannot change state.mp after aux is initialized.')
        else:
            self.p = self.new_array(mp)

    @property
    def mF(self):
        r"""(int) - Number of output functionals"""
        if self.F is not None: return self.F.shape[0]
        else: return 0
    @mF.setter
    def mF(self,mF):
        if self.F is not None:
            raise Exception('Cannot change state.mF after aux is initialized.')
        else:
            self.F = self.new_array(mF)

    # ========== Class Methods ===============================================
    def __init__(self,geom,num_eqn,num_aux=0):
        from clawpack.pyclaw import geometry
        if isinstance(geom,geometry.Patch):
            self.patch = geom
        elif isinstance(geom,geometry.Domain):
            self.patch = geom.patches[0]
        else:
            raise Exception("""A PyClaw State object must be initialized with
                             a PyClaw Patch object.""")

        # ========== Attribute Definitions ===================================
        r"""pyclaw.Patch.patch - The patch this state lives on"""
        self.p   = None
        r"""(ndarray(mp,...)) - Cell averages of derived quantities."""
        self.F   = None
        r"""(ndarray(mF,...)) - Cell averages of output functional densities."""
        self.problem_data = {}
        r"""(dict) - Dictionary of global values for this patch, 
            ``default = {}``"""
        self.t=0.
        r"""(float) - Current time represented on this patch, 
            ``default = 0.0``"""
        self.index_capa = -1
        self.keep_gauges = False
        r"""(bool) - Keep gauge values in memory for every time step, 
        ``default = False``"""
        self.gauge_data = []
        r"""(list) - List of numpy.ndarray objects. Each element of the list
        stores the values of the corresponding gauge if ``keep_gauges`` is set
        to ``True``"""
        

        self.q   = self.new_array(num_eqn)
        self.aux = self.new_array(num_aux)

    def __str__(self):
        output = "PyClaw State object\n"
        output += "Patch dimensions: %s\n" % str(self.patch.num_cells_global)
        output += "Time  t=%s\n" % (self.t)
        output += "Number of conserved quantities: %s\n" % str(self.q.shape[0])
        if self.aux is not None:
            output += "Number of auxiliary fields: %s\n" % str(self.aux.shape[0])
        if self.problem_data != {}:
            output += "problem_data: "+self.problem_data.__str__()
        return output

    def is_valid(self):
        r"""
        Checks to see if this state is valid
        
        The state is declared valid based on the following criteria:
            - :attr:`q` is Fortran contiguous
            - :attr:`aux` is Fortran contiguous
            
        A debug logger message will be sent documenting exactly what was not 
        valid.
            
        :Output:
         - (bool) - True if valid, false otherwise.
        
        """
        import logging
        valid = True
        logger = logging.getLogger('pyclaw.solution')
        if not self.q.flags['F_CONTIGUOUS']:
            logger.debug('q array is not Fortran contiguous.')
            valid = False
        if self.aux is not None:
            if not self.aux.flags['F_CONTIGUOUS']:
                logger.debug('aux array is not Fortran contiguous.')
                valid = False
        return valid
 
    def set_cparam(self,fortran_module):
        """
        Set the variables in fortran_module.cparam to the corresponding values in
        patch.problem_data.  This is the mechanism for passing scalar variables to the
        Fortran Riemann solvers; cparam must be defined as a common block in the
        Riemann solver.

        This function should be called from solver.setup().  This seems like a fragile
        interdependency between solver and state; perhaps problem_data should belong
        to solver instead of state.

        This function also checks that the set of variables defined in cparam 
        all appear in problem_data.
        """
        if hasattr(fortran_module,'cparam'):
            try:
                paramlist = [parm for parm in fortran_module.cparam.__dir__()
                             if '__' not in parm]
            except AttributeError: # Python 2
                paramlist = dir(fortran_module.cparam)
            if not set(paramlist) <= set(self.problem_data.keys()):
                raise Exception("""Some required value(s) in the cparam common 
                                   block in the Riemann solver have not been 
                                   set in problem_data.""")
            for global_var_name,global_var_value in self.problem_data.items():
                setattr(fortran_module.cparam,global_var_name,global_var_value)

    def set_num_ghost(self,num_ghost):
        """
        Virtual routine (does nothing).  Overridden in the petclaw.state class.
        """
        pass

    def set_q_from_qbc(self,num_ghost,qbc):
        """
        Set the value of q using the array qbc. Typically this is called
        after qbc has been updated by the solver.
        """
        
        num_dim = self.patch.num_dim
        if num_dim == 1:
            self.q = qbc[:,num_ghost:-num_ghost]
        elif num_dim == 2:
            self.q = qbc[:,num_ghost:-num_ghost,num_ghost:-num_ghost]
        elif num_dim == 3:
            self.q = qbc[:,num_ghost:-num_ghost,num_ghost:-num_ghost,num_ghost:-num_ghost]
        else:
            raise Exception("Assumption (1 <= num_dim <= 3) violated.")
            
    def set_aux_from_auxbc(self,num_ghost,auxbc):
        """
        Set the value of aux using the array auxbc. 
        """
        
        patch = self.patch
        if patch.num_dim == 1:
            self.aux = auxbc[:,num_ghost:-num_ghost]
        elif patch.num_dim == 2:
            self.aux = auxbc[:,num_ghost:-num_ghost,num_ghost:-num_ghost]
        elif patch.num_dim == 3:
            self.aux = auxbc[:,num_ghost:-num_ghost,num_ghost:-num_ghost,num_ghost:-num_ghost]
        else:
            raise Exception("Assumption (1 <= num_dim <= 3) violated.")


    def get_qbc_from_q(self,num_ghost,qbc):
        """
        Fills in the interior of qbc by copying q to it.
        """
        num_dim = self.patch.num_dim
        
        if num_dim == 1:
            qbc[:,num_ghost:-num_ghost] = self.q
        elif num_dim == 2:
            qbc[:,num_ghost:-num_ghost,num_ghost:-num_ghost] = self.q
        elif num_dim == 3:
            qbc[:,num_ghost:-num_ghost,num_ghost:-num_ghost,num_ghost:-num_ghost] = self.q
        else:
            raise Exception("Assumption (1 <= num_dim <= 3) violated.")

        return qbc
        
    def get_auxbc_from_aux(self,num_ghost,auxbc):
        """
        Fills in the interior of auxbc by copying aux to it.
        """
        num_dim = self.patch.num_dim
        
        if num_dim == 1:
            auxbc[:,num_ghost:-num_ghost] = self.aux
        elif num_dim == 2:
            auxbc[:,num_ghost:-num_ghost,num_ghost:-num_ghost] = self.aux
        elif num_dim == 3:
            auxbc[:,num_ghost:-num_ghost,num_ghost:-num_ghost,num_ghost:-num_ghost] = self.aux
        else:
            raise Exception("Assumption (1 <= num_dim <= 3) violated.")

        return auxbc
        

    # ========== Copy functionality ==========================================
    def __copy__(self):
        return self.__class__(self)
        
    def __deepcopy__(self,memo={}):
        import copy
        result = self.__class__(copy.deepcopy(self.patch),self.num_eqn,self.num_aux)
        result.__init__(copy.deepcopy(self.patch),self.num_eqn,self.num_aux)
        
        for attr in ('t'):
            setattr(result,attr,copy.deepcopy(getattr(self,attr)))
        
        if self.q is not None:
            result.q = copy.deepcopy(self.q)
        if self.aux is not None:
            result.aux = copy.deepcopy(self.aux)
        result.problem_data = copy.deepcopy(self.problem_data)
        
        return result

    def sum_F(self,i):
        import numpy as np
        return np.sum(np.abs(self.F[i,...]))

    def new_array(self,dof):
        import numpy as np
        if dof==0: return None
        shape = [dof]
        shape.extend(self.grid.num_cells)
        return np.empty(shape,order='F')

    def get_q_global(self):
        r"""
        Returns a copy of state.q.
        """
        return self.q.copy()

    def get_aux_global(self):
        r"""
        Returns a copy of state.aux.
        """
        return self.aux.copy()

if __name__ == "__main__":
    import doctest
    doctest.testmod()



"""
Module for PetClaw controller class.
"""

from __future__ import absolute_import
from clawpack import pyclaw

class Controller(pyclaw.controller.Controller):
    """ Parallel Controller Class

    Defaults to petsc output_format, logs only from process 0.
    """
    
    __doc__ += pyclaw.util.add_parent_doc(pyclaw.controller.Controller)

    def __init__(self):
        super(Controller,self).__init__()

        self.output_format = 'petsc'

    def is_proc_0(self):
        from petsc4py import PETSc
        rank = PETSc.Comm.getRank(PETSc.COMM_WORLD)
        return rank == 0

    def log_info(self, str):
        import logging
        if self.is_proc_0():
            logging.info(str)
        else:
            pass



r"""(Py)WENO based reconstructor for hyperbolic PDEs.

The :py:mod:`weno.reconstruct` module needs to be built before this
module can be used.  See 'weno/codegen.py' for details.

To build a higher order reconstruction, *k* needs to be tweaked here
and in 'weno/codegen.py'.  Also, *num_ghost* needs to be tweaked in the
PyClaw solver.

"""

import pyclaw.limiters.weno.reconstruct as recon

def weno(k, q):
    r"""Return the *k* order WENO based reconstruction of *q*.

    The reconstruction is component based.
    """

    import numpy as np
    # XXX: this should really by a class so that the workspaces (sigma
    # and weights) can be pre-allocated and the 'getattr's can be done
    # once instead of every call

    if (k % 2) == 0:
        raise ValueError('even order WENO reconstructions are not supported')

    k = (k+1)//2
    sigma = np.zeros((q.shape[1], k))
    weights = np.zeros((q.shape[1], k))

    ql = np.zeros(q.shape)
    qr = np.zeros(q.shape)

    try:
        smoothness    = getattr(recon, 'smoothness_k' + str(k))
        weights_l     = getattr(recon, 'weights_left_k' + str(k))
        weights_r     = getattr(recon, 'weights_right_k' + str(k))
        reconstruct_l = getattr(recon, 'reconstruct_left_k' + str(k))
        reconstruct_r = getattr(recon, 'reconstruct_right_k' + str(k))
    except:
        raise ValueError('%d order WENO reconstructions are not supported' % (2*k-1))


    for m in range(q.shape[0]):
        smoothness(q[m,:], sigma)

        weights_l(sigma, weights)
        reconstruct_l(q[m,:], weights, ql[m,:])

        weights_r(sigma, weights)
        reconstruct_r(q[m,:], weights, qr[m,:])


    # XXX: copy ghost-cells.  i'm not sure why this is necessary, but
    # it make the acoustics examples in the implicit time-stepping
    # branch work properly.

    ql[:,:k-1]  = ql[:,-2*k+2:-k+1]
    ql[:,-k+1:] = ql[:,k-1:2*k-2]

    qr[:,:k-1]  = qr[:,-2*k+2:-k+1]
    qr[:,-k+1:] = qr[:,k-1:2*k-2]

    return ql, qr



#!/usr/bin/env python
# encoding: utf-8
    
from __future__ import absolute_import
def fig_31_38(iplot=False,htmlplot=False,outdir='./_output'):
    r"""Produces the output shown in Figures 3.1 and 3.8 of the FVM book.
    These involve simple waves in the acoustics system."""
    from clawpack import pyclaw
    from clawpack import riemann
    import numpy as np

    solver = pyclaw.ClawSolver1D(riemann.acoustics_1D)

    solver.limiters = pyclaw.limiters.tvd.MC
    solver.bc_lower[0] = pyclaw.BC.wall
    solver.bc_upper[0] = pyclaw.BC.extrap

    x = pyclaw.Dimension(-1.0,1.0,800,name='x')
    domain = pyclaw.Domain(x)
    num_eqn = 2
    state = pyclaw.State(domain, num_eqn)

    # Set problem-specific variables
    rho = 1.0
    bulk = 0.25
    state.problem_data['rho']=rho
    state.problem_data['bulk']=bulk
    state.problem_data['zz']=np.sqrt(rho*bulk)
    state.problem_data['cc']=np.sqrt(bulk/rho)

    # Set the initial condition
    xc = domain.grid.x.centers
    beta=100; gamma=0; x0=0.75
    state.q[0,:] = 0.5*np.exp(-80 * xc**2) + 0.5*(np.abs(xc+0.2)<0.1)
    state.q[1,:] = 0.
    
    # Set up the controller
    claw = pyclaw.Controller()
    claw.solution = pyclaw.Solution(state, domain)
    claw.solver = solver
    claw.outdir = outdir
    claw.tfinal = 3.0
    claw.num_output_times   = 30

    return claw

if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(fig_31_38)



"""
Make stand-alone animations similar to the movies in the plots directory.

First run `make plots` so that directory exists with png files for the
individual frames.

For illustration of other tools defined in the animation_tools module, 
see the notebook
  $CLAW/apps/notebooks/visclaw/animation_tools_demo.html
also visible in rendered form in the Clawpack gallery:
  http://www.clawpack.org/gallery/_static/apps/notebooks/visclaw/animation_tools_demo.html
  
"""

import matplotlib
matplotlib.use('Agg')
import os

from clawpack.visclaw import animation_tools

# file name prefix obtained from this directory name:
this_dir = os.path.split(os.getcwd())[-1]
file_name_prefix = this_dir + '_'

# Plots directory:  (first run `make plots`)
plotdir = '_plots'

# Search for all movie files in plotdir to determine what movies to make:
fignos = 'all'

# The type of outputs desired (from 'mp4', 'html', 'rst', the latter for use
# with Sphinx documentation (reStructured text):
outputs = ['html','rst','mp4']

# Size of figure:
figsize = (5,4)

# Dots per inch:
dpi = None

# Frames per second (for mp4 file only):
fps = 5

animation_tools.make_anim_outputs_from_plotdir(plotdir=plotdir, fignos=fignos,
        outputs=outputs, file_name_prefix=file_name_prefix,
        figsize=figsize, dpi=dpi, fps=fps)



"""
A tool to add a desired legend to an existing plot more easily when 
adaptive mesh refinement is used in VisClaw.

To do fancier things with legends, see:
    http://matplotlib.org/users/legend_guide.html
"""

def add_legend(labels,colors='k',linestyles='-',markers='', **kwargs):
    """
    Add a legend to an existing plot with a specified set of line colors.
    Input:
        labels:  list of labels
        colors:  list of colors, of same length as labels
            or a single string used for all entries.
            Default is 'k', so all will be black.
        linestyles: list of strings of same length as labels, 
            or a single string used for all entries.
            Default is '-', so all will be solid lines. 
            Set to '' for no line.
        markers: list of strings of same length as labels,  
            or a single string used for all entries.
            Default is '', so no marker for any entry
        kwargs: keyword arguments to be passed to `plt.legend`, e.g.
            `loc` or `fontsize`.
    There is one entry in the legend for each color/label.
    
    Useful for annotating an AMR plot when plots were made
    by looping over patches.  In this case it's not possible to simply add
    a label as a kwarg to the plot command or the legend will contain an
    entry for every patch.    
    """
    
    import matplotlib.lines as mlines
    import matplotlib.pyplot as plt
    
    # convert single string to list of correct length:
    
    if type(colors) is str:
        colors = len(labels)*[colors]

    if type(linestyles) is str:
        linestyles = len(labels)*[linestyles]

    if type(markers) is str:
        markers = len(labels)*[markers]
            
    handles = []
    for k in range(len(colors)):
          handle = mlines.Line2D([], [], color=colors[k], 
                    linestyle=linestyles[k], marker=markers[k], 
                    label=labels[k])
          handles.append(handle)
              
    plt.legend(handles=handles,**kwargs)
    



#!/usr/bin/env python
# encoding: utf-8

""" 
Test problem demonstrating 3D hot-sphere rising in an stabilized atmosphere
   in Cartesian coordinates

This problem evolves the 3D Euler equations using an F-wave
   method, with gravitational source term modifications.
The primary variables are: 
   density (rho), x,y, and z momentum (rho*u,rho*v,rho*w), and energy.
"""
import numpy as np
from clawpack.riemann.mappedGrid import euler3d_mappedgrid as mg

try:
    from mpi4py import MPI
    mpiAvailable = True
except ImportError:
    import warnings
    warnings.warn('mpi4py is not available')
    mpiAvailable = False

if mpiAvailable:
    mpiRank = MPI.COMM_WORLD.Get_rank()
    mpiSize = MPI.COMM_WORLD.Get_size()
else:
    mpiRank = 0
    mpiSize = 1

# Constants
gamma = 1.4 # Ratio of specific heats
gamma1 = gamma - 1.
gR = 980.665 # Acceleration due to gravity [cm/s**2]
kBoltzmann = 1.3807e-16 # Boltzmann constant [erg/K]
nAvogadro = 6.0221e23 # Avogadro's number [1/mol]

# Hot Sphere Parameters
xSphere = 1000.e5; ySphere = 1000.e5; zSphere = 150.e5
rSphere = 40.e5 # Radius of Sphere [cm]
TSphere = 2.7e4 # Temperature of Sphere Perturbation

# Grid Parameters
mxyz = [80,80,80] # Number of Grid Cells
xyzMin = [500.e5  , 500.e5  , 80.0e5 ] # Domain limits (min) [cm]
xyzMax = [1500.0e5, 1500.0e5, 950.0e5] # Domain limits (max) [cm]
mapType = "ZeroToOne"
z0 = xyzMin[2]
zN = xyzMax[2]

# Gravity Terms
gravityTerm = True # Turn Gravity Term On or Off in Riemann Solver
gravityEflux = False # Turn Gravity Term in Energy Flux On/Off
gFlux = 0
if gravityEflux: gFlux = 1

#-----------------------------------------------------------------------
# Name: outputDensity(state)
#
# Description:
#   Using derived quantities function, output 2D slice of density
#   for use in regression testing.
#-----------------------------------------------------------------------
def outputDensity(state):
    state.p[0,:,:,:] = state.q[0,:,:,:]
    nx = np.size(state.q,1)
    x = np.reshape(state.q[0,nx/2,:,:],np.size(state.q[0,nx/2,:,:]),order='F')
    np.savetxt('verify_rising_hot_sphere_classic_1.txt',x,fmt='%18.8e',delimiter=' ')

#-----------------------------------------------------------------------
# Description:
#   Equilibrium atmosphere
#
# Inputs:
#   p0[mz+2*mbc]      : pressure (1D array)
#   rho0[mz+2*mbc]    : density (1D array)
#   Mavg[mz+2*mbc]    : average molecular mass (1D array)
# 
# Input/Outputs:
#   p0,rho0,Mavg      : 1D z-column initialization of p0 and rho0
#-----------------------------------------------------------------------
def setEquilibriumAtmosphere(p0,rho0,Mavg):

    p0 = [1.28255457e+02,2.45768842e+01,4.14947876e+00,6.29750420e-01,1.01220380e-01,2.64133921e-02,1.22941741e-02,7.08667395e-03,4.52931611e-03,3.07286214e-03,2.16905463e-03,1.57652477e-03,1.17092484e-03,8.84611067e-04,6.77691403e-04,5.25138237e-04,4.10841768e-04,3.24102394e-04,2.57470120e-04,2.05925021e-04,1.65598592e-04,1.33701518e-04,1.08364754e-04,8.82441931e-05,7.21143717e-05,5.91376054e-05,4.86178229e-05,4.00787900e-05,3.30908693e-05,2.73888126e-05,2.27031016e-05,1.88518481e-05,1.56898948e-05,1.30700401e-05,1.08991559e-05,9.09869161e-06,7.60521743e-06,6.36376491e-06,5.32972657e-06,4.46856235e-06,3.74878325e-06,3.14890785e-06,2.64613146e-06,2.22646032e-06,1.87396531e-06,1.57844875e-06,1.33028392e-06,1.12211091e-06,9.47071388e-07,7.99762122e-07,6.75921511e-07,5.71493939e-07,4.83610358e-07,4.09325094e-07,3.46744110e-07,2.93793938e-07,2.49152408e-07,2.11367113e-07,1.79432411e-07,1.52415843e-07,1.29549499e-07,1.10136422e-07,9.37086690e-08,7.97324669e-08,6.79127210e-08,5.78532722e-08,4.93172661e-08,4.20604343e-08,3.58836884e-08,3.06389102e-08,2.61608771e-08,2.23557534e-08,1.91042726e-08,1.63479490e-08,1.39976779e-08,1.19853352e-08,1.02623231e-08,8.78713846e-09,7.53940212e-09,6.46885245e-09,5.55032464e-09,4.76222864e-09,4.09020086e-09,3.51658796e-09]

    rho0 = [1.93347036e-07,4.03984315e-08,7.33795328e-09,1.16964004e-09,1.64049100e-10,2.53990286e-11,7.54287116e-12,3.40478277e-12,1.84556481e-12,1.10964372e-12,7.13581470e-13,4.81506393e-13,3.36472592e-13,2.41540079e-13,1.77156053e-13,1.32213794e-13,1.00089557e-13,7.67024111e-14,5.93930647e-14,4.64294817e-14,3.65782332e-14,2.90138753e-14,2.31378048e-14,1.85800114e-14,1.49929512e-14,1.21526733e-14,9.89015561e-15,8.07840567e-15,6.61976992e-15,5.43890503e-15,4.48202167e-15,3.70250573e-15,3.06590093e-15,2.54266886e-15,2.11283102e-15,1.75827860e-15,1.46560471e-15,1.22337830e-15,1.02239821e-15,8.55585508e-16,7.16578299e-16,6.01033981e-16,5.04419184e-16,4.23940996e-16,3.56468062e-16,2.99992883e-16,2.52633808e-16,2.12955966e-16,1.79630105e-16,1.51610996e-16,1.28075790e-16,1.08244792e-16,9.15665290e-17,7.74771188e-17,6.56137471e-17,5.55805979e-17,4.71251502e-17,3.99708405e-17,3.39261636e-17,2.88137888e-17,2.44878021e-17,2.08159094e-17,1.77092661e-17,1.50666724e-17,1.28321441e-17,1.09306468e-17,9.31730480e-18,7.94587120e-18,6.77866202e-18,5.78764327e-18,4.94156316e-18,4.22266806e-18,3.60840539e-18,3.08771188e-18,2.64374425e-18,2.26362608e-18,1.93817162e-18,1.65953699e-18,1.42386938e-18,1.22167290e-18,1.04819271e-18,8.99349679e-19,7.72429901e-19,6.64098458e-19]

    Mavg = [28.85614554,28.85337155,28.83817654,28.56226512,27.60224909,26.26692289,25.23573593,24.45469565,23.79308533,23.18781005,22.61490394,22.07318988,21.55703223,21.06778441,20.60540309,20.17202267,19.76585711,19.38847601,19.0408475, 18.71970337,18.42758099,18.16274099,17.92359740,17.70606183,17.51035814,17.33530373,17.17893585,17.03979933,16.91620578,16.80712079,16.71028376,16.62471452,16.54940299,16.48292773,16.42454596,16.37307369,16.32776306,16.28801338,16.2531155, 16.22247335,16.19551611,16.17188138,16.15108306,16.13288090,16.11686426,16.10282002,16.09046507,16.07960946,16.07007411,16.06169374,16.05433222,16.04784993,16.04215209,16.03712679,16.0327204,16.02883120,16.02540929,16.02239140,16.01973516,16.01738918,16.01531699,16.01348647,16.01187781,16.01045286,16.00919766,16.00808580,16.00710454,16.00623687,16.00546792,16.00478755,16.00418349,16.00365220,16.00317996,16.00276269,16.00239247,16.00206303,16.00176987,16.00150902,16.00127962,16.00107519,16.00089299,16.00073063,16.00058692,16.00045964]

    return p0,rho0,Mavg

#-----------------------------------------------------------------------
# Description:
#   Modify pressure to create numeric atmosphere equilibrium
#
# Inputs:
#   ze0[mz+2*mbc+1]     : cell edge grid values
#   p0[mz+2*mbc]        : pressure
#   rho0[mz+2*mbc]      : density
#
# Input/Outputs:
#   p0,rho0           : 1D z-column modification of p0 and rho0
#-----------------------------------------------------------------------
def modifyEquilibriumAtmosphere(zep0,p0,rho0):

    # Compute the delta-z (dz)
    nz = np.size(zep0)-1
    dz = np.zeros([nz],dtype='float',order='F')
    for iz in range(nz-1):
        dz[iz] = zep0[iz+1]-zep0[iz]

    # Compute modified pressure at cell centers
    iz = nz-1
    dz2 = (dz[iz]+dz[iz-1])*0.5
    p0[iz] = p0[iz] + rho0[iz]*gR*dz2
    for iz in range(nz-1,0,-1):
        dz2 = (dz[iz]+dz[iz-1])*0.5
        finterp = dz[iz-1]/(dz[iz]+dz[iz-1])
        rho_b = rho0[iz]*finterp + rho0[iz-1]*(1.-finterp)
        p0[iz-1] = p0[iz] + rho_b*gR*dz2

    return p0

#-----------------------------------------------------------------------
# Description:
#   Custom BCs for the z-direction
#-----------------------------------------------------------------------
def customBCLowerZ(state,dim,t,qbc,auxbc,mbc):
    for k in range(mbc):
        qbc[0,:,:,k] =  rho0[k]
        qbc[1,:,:,k] =  0.
        qbc[2,:,:,k] =  0.
        qbc[3,:,:,k] =  0.
        qbc[4,:,:,k] =  p0[k]/gamma1 + qbc[0,:,:,k]*gR*zcpZ[k]*gFlux

def customBCUpperZ(state,dim,t,qbc,auxbc,mbc):
    for k in range(mbc):
        qbc[0,:,:,-k-1] = rho0[-k-1]
        qbc[1,:,:,-k-1] = qbc[1,:,:,-mbc-1]
        qbc[2,:,:,-k-1] = qbc[2,:,:,-mbc-1]
        qbc[3,:,:,-k-1] = qbc[3,:,:,-mbc-1]
        rhov2 = (qbc[1,:,:,-k-1]**2 + qbc[2,:,:,-k-1]**2 + qbc[3,:,:,-k-1]**2)/qbc[0,:,:,-k-1]
        qbc[4,:,:,-k-1] = p0[-k-1]/gamma1 + 0.5*rhov2 + qbc[0,:,:,-k-1]*gR*zcpZ[-k-1]*gFlux

def customAuxBCLowerZ(state,dim,t,qbc,auxbc,mbc):
    auxbc[:,:,:,:mbc] = auxtmp[:,:,:,:mbc]

def customAuxBCUpperZ(state,dim,t,qbc,auxbc,mbc):
    auxbc[:,:,:,-mbc:] = auxtmp[:,:,:,-mbc:]

#-----------------------------------------------------------------------
#  Main script for solving 3D Euler equations using Clawpack/PyClaw.
#-----------------------------------------------------------------------
def euler3d(kernel_language='Fortran',solver_type='classic',\
            use_petsc=False,outdir='./_output',\
            output_format='hdf5',file_prefix='equil',disable_output=False,\
            mx=mxyz[0],my=mxyz[1],mz=mxyz[2],\
            tfinal=64.0,num_output_times=1,keep_copy=False):

    if use_petsc:
        import clawpack.petclaw as pyclaw
    else:
        from clawpack import pyclaw

    if solver_type=='classic':
        solver = pyclaw.ClawSolver3D()
        solver.dimensional_split = True
        solver.limiters = pyclaw.limiters.tvd.minmod
        solver.num_ghost = 2
        solver.order = 2
        solver.fwave = True
    elif solver_type=='sharpclaw':
        solver = pyclaw.SharpClawSolver3D()
    else:
        raise Exception('Unrecognized solver_type.')

    import logging
    solver.logger.setLevel(logging.DEBUG)
    from clawpack import riemann
    solver.rp = riemann.euler_mapgrid_3D
    solver.num_eqn = 5
    solver.num_waves = 3
    solver.cfl_max = 0.6
    solver.cfl_desired = 0.5
    solver.dt_initial = 1.e-0
    solver.max_steps = 10000

    # Initialize Domain
    x = pyclaw.Dimension(0.0,1.0,mx,name='x')
    y = pyclaw.Dimension(0.0,1.0,my,name='y')
    z = pyclaw.Dimension(0.0,1.0,mz,name='z')
    domain = pyclaw.Domain([x,y,z])

    num_aux = 15
    state = pyclaw.State(domain,solver.num_eqn,num_aux)
    state.problem_data['gamma']=gamma
    state.problem_data['g_r'] = gR
    state.problem_data['gravity'] = gravityTerm
    state.problem_data['gravityflux'] = gravityEflux

    # Grids
    mbc = solver.num_ghost
    grid = state.grid

    # Computational Grid Sizes
    dxc = domain.grid.delta[0]
    dyc = domain.grid.delta[1]
    dzc = domain.grid.delta[2]
    pmx, pmy, pmz = grid.num_cells[0], grid.num_cells[1], grid.num_cells[2]

    # Computational Grid Centers and nodes
    centers = grid.c_centers # centers (Comp.)
    centersBC = grid.c_centers_with_ghost(mbc) # centers w Ghost (Comp.)
    nodesBC = grid.c_nodes_with_ghost(mbc) # nodes w Ghost (Comp.)

    # Grid Centers Without Boundary Cells (1D Slice) - Comp. and Phys.
    xcc = grid.x.centers # x centers (Comp.)
    ycc = grid.y.centers # y centers (Comp.)
    zcc = grid.z.centers # z centers (Comp.)
    xcp,ycp,zcp = mg.mapc2pwrapper(xcc,ycc,zcc,pmz,xyzMin,xyzMax,mapType)

    # Grid Centers Without Boundary Cells (3D Arrays)
    Xcc,Ycc,Zcc = centers[0][:][:][:],centers[1][:][:][:],centers[2][:][:][:]
    Xcp,Ycp,Zcp = mg.mapc2pwrapper(Xcc,Ycc,Zcc,pmz,xyzMin,xyzMax,mapType)
    Xcp = np.reshape(Xcp,[pmx,pmy,pmz],order='F') # x centers (Phys.)
    Ycp = np.reshape(Ycp,[pmx,pmy,pmz],order='F') # y centers (Phys.)
    Zcp = np.reshape(Zcp,[pmx,pmy,pmz],order='F') # z centers (Phys.)

    # Grid nodes With Boundary Cells (1D Slice along z)- Comp. and Phys.
    xecZ = nodesBC[0][0][0][:] # x nodes along z (Comp.)
    yecZ = nodesBC[1][0][0][:] # y nodes along z (Comp.)
    zecZ = nodesBC[2][0][0][:] # z nodes along z (Comp.)
    xepZ,yepZ,zepZ = mg.mapc2pwrapper(xecZ,yecZ,zecZ,pmz,xyzMin,xyzMax,mapType)

    # Grid Centers With Boundary Cells (1D Slice along z) - Comp. and Phys.
    global zcpZ
    xccZ = centersBC[0][0][0][:] # x centers along z (Comp.)
    yccZ = centersBC[1][0][0][:] # y centers along z (Comp.)
    zccZ = centersBC[2][0][0][:] # z centers along z (Comp.)
    xcpZ,ycpZ,zcpZ = mg.mapc2pwrapper(xccZ,yccZ,zccZ,pmz,xyzMin,xyzMax,mapType)
 
    if np.sqrt(xepZ[0]**2+yepZ[0]**2+zepZ[0]**2) <= 0:
        print("WARNING: z may go below Earth's surface"," zepZ: ",zepZ[0:10])

    # Create vectors for 1D pressure and density column with boundary cells
    mz0 = pmz+2*mbc
    global p0, rho0, Mavg
    p0 = np.zeros([mz0],dtype='float',order='F')
    rho0 = np.zeros([mz0],dtype='float',order='F')
    Mavg = np.zeros([mz0],dtype='float',order='F')

    # Set the equilibrium pressure such that dp/dz = -rho*gR
    p0,rho0,Mavg = setEquilibriumAtmosphere(p0,rho0,Mavg)

    # Modify the equilibrium such that dp/dz = -rho*gR is held numerically
    p0 = modifyEquilibriumAtmosphere(zepZ,p0,rho0)

    # Set the auxiliary variables
    xlower,ylower,zlower = nodesBC[0][0][0][0],nodesBC[1][0][0][0],nodesBC[2][0][0][0]
    dxc,dyc,dzc = domain.grid.delta[0],domain.grid.delta[1],domain.grid.delta[2]
        
    global auxtmp
    auxtmp = np.zeros([num_aux,pmx+2*mbc,pmy+2*mbc,pmz+2*mbc],dtype='float',order='F')
    auxtmp = mg.setauxiliaryvariables(num_aux,mbc,pmx,pmy,pmz,xlower,ylower,zlower,dxc,dyc,dzc,xyzMin,xyzMax,mapType)
    state.aux[:,:,:,:] = auxtmp[:,mbc:-mbc,mbc:-mbc,mbc:-mbc]

    # Set Index for Capcaity Function in state.aux (Python 0-based)
    state.index_capa = 12 

    # Set the state variables (Initial Conditions)

    # Initialize p,T,velSqrd
    p = np.zeros([pmx,pmy,pmz],dtype='float',order='F')
    T = np.zeros([pmx,pmy,pmz],dtype='float',order='F')
    velSqrd = np.zeros([pmx,pmy,pmz],dtype='float',order='F')

    # Density
    for i in range(pmx):
        for j in range(pmy):
            # NEEDS TO BE FIXED WHEN MPI SLICES NORMAL TO Z
            state.q[0,i,j,:] = rho0[mbc:pmz+mbc]
    
    # Momentum
    state.q[1,:,:,:] = 0. # x-momentum (rho*u)
    state.q[2,:,:,:] = 0. # y-momentum (rho*v)
    state.q[3,:,:,:] = 0. # z-momentum (rho*w)

    # Velocity Squared (u**2+v**2+w**2)
    velSqrd[:,:,:] = (state.q[1,:,:,:]**2+state.q[2,:,:,:]**2 + state.q[3,:,:,:]**2)/state.q[0,:,:,:]**2

    # Energy
    for i in range(pmx):
        for j in range(pmy):
            # NEEDS TO BE FIXED WHEN MPI SLICES NORMAL TO Z
            p[i,j,:] = p0[mbc:pmz+mbc]
    state.q[4,:,:,:] = p/gamma1 + 0.5*state.q[0,:,:,:]*velSqrd + state.q[0,:,:,:]*(gR)*Zcp[:,:,:]*gFlux

    # Add Temperature Perturbation
    T = p/state.q[0,:,:,:]
    L = np.sqrt((Xcp-xSphere)**2+(Ycp-ySphere)**2+(Zcp-zSphere)**2)
    for i in range(pmx):
        for j in range(pmy):
            for k in range(pmz):
                if L[i,j,k] <= rSphere:
                    mu = Mavg[k+mbc]/nAvogadro
                    T[i,j,k] += TSphere*(kBoltzmann/mu)*(1.0-L[i,j,k]/rSphere)
                    p[i,j,k] = T[i,j,k]*state.q[0,i,j,k]
    state.q[4,:,:,:] = p/gamma1 + 0.5*state.q[0,:,:,:]*velSqrd + state.q[0,:,:,:]*(gR)*Zcp[:,:,:]*gFlux # energy (e)

    # Setup Boundary Conditions

    # X - Boundary Conditions
    solver.bc_lower[0] = pyclaw.BC.extrap
    solver.bc_upper[0] = pyclaw.BC.extrap

    # Y - Boundary Conditions
    solver.bc_lower[1] = pyclaw.BC.extrap
    solver.bc_upper[1] = pyclaw.BC.extrap

    # Z - Boundary Conditions
    solver.bc_lower[2] = pyclaw.BC.custom
    solver.bc_upper[2] = pyclaw.BC.custom
    solver.user_bc_lower = customBCLowerZ
    solver.user_bc_upper = customBCUpperZ

    # Aux - Boundary Conditions
    solver.aux_bc_lower[0] = pyclaw.BC.extrap
    solver.aux_bc_upper[0] = pyclaw.BC.extrap
    solver.aux_bc_lower[1] = pyclaw.BC.extrap
    solver.aux_bc_upper[1] = pyclaw.BC.extrap
    solver.aux_bc_lower[2] = pyclaw.BC.custom
    solver.aux_bc_upper[2] = pyclaw.BC.custom
    solver.user_aux_bc_lower = customAuxBCLowerZ
    solver.user_aux_bc_upper = customAuxBCUpperZ

    # Solver Parameters
    claw = pyclaw.Controller()
    claw.verbosity = 4
    claw.solution = pyclaw.Solution(state,domain)
    claw.solver = solver
    claw.output_format = output_format
    claw.output_file_prefix = file_prefix
    claw.keep_copy = keep_copy
    if disable_output:
        claw.output_format = None
    claw.tfinal = tfinal
    claw.num_output_times = num_output_times
    claw.outdir = outdir
    #state.mp = 1
    #claw.compute_p = outputDensity

    return claw

# __main__()
if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(euler3d)



from __future__ import absolute_import
from __future__ import print_function
from mpi4py import MPI  
comm = MPI.COMM_WORLD 
size = comm.Get_size()
rank = comm.Get_rank()
x = rank
print('x before', x)
max_x =comm.reduce( sendobj=x, op=MPI.MAX,  root=0)
x = comm.bcast(max_x, root=0)
print('x after', x)





"""
Tools to assist interpolating from fort.gauge output to a given
set of times (e.g. equally spaced).

Example:
  Set gaugeno to the desired gauge number, iq to the component of q to 
  plot, e.g. iq=3 for surface eta.

    import gauge_interp
    getgauge = gauge_interp.make_getgauge(output='_outdir')
    gf,t1,t2 = gauge_interp.gauge_function(getgauge, gaugeno, iq)
    t = numpy.linspace(t1,t2,50)  # desired times of interpolation
    gt = gf(t)
    
Then gt will be an array of the same length as t containing interpolated 
values (piecewise linear).

See also the test function below, which can be run via:

    $ cd $CLAW/geoclaw/examples/tsunami/chile2010
    $ make .output
    $ python
    >>> from clawpack.visclaw import gauge_interp
    >>> gauge_interp.test()

"""


from __future__ import absolute_import
def make_getgauge(outdir='_output'):
    """
    Create function getgauge that will grab one set of gauge data
    from the fort.gauge file in directory specified by outdir.
    """
    from clawpack.visclaw.data import ClawPlotData
    plotdata = ClawPlotData()
    plotdata.outdir = outdir
    getgauge = plotdata.getgauge
    return getgauge
    

def gauge_function(getgauge, gaugeno, iq):
    """
    Given getgauge function, return a function that
    can be used to evaluate the iq component of q at any time
    in the interval covered by the gauge output.

    Also returns t1, t2 the end points of validity.
    
    Does piecewise linear interpolation based on g.q and g.q[iq,:]
    where g is the gauge data.
    """

    from scipy import interpolate

    g = getgauge(gaugeno)
    gf = interpolate.interp1d(g.t, g.q[iq,:])
    t1 = g.t[0]
    t2 = g.t[-1]
    return gf, t1, t2


def test(gaugeno=32412, iq=3):

    # Test should work in $CLAW/geoclaw/examples/tsunami/chile2010
    import numpy
    import matplotlib.pyplot as plt

    # interpolate at a set of times t:
    getgauge = make_getgauge(outdir='_output')
    gf,t1,t2 = gauge_function(getgauge, gaugeno, iq)
    t = numpy.linspace(t1,t2,100)
    gt = gf(t)

    # Plot the two for comparison:
    plt.plot(t,gt,'ro', markersize=5, label='interpolated')
    g = getgauge(gaugeno)
    plt.plot(g.t, g.q[iq,:], 'k-',label='original data')
    plt.legend()
    plt.show()




"""
Create topo and dtopo files needed for this example:
    etopo10min120W60W60S0S.asc        download from GeoClaw topo repository
    dtopo_usgs100227.tt3              create using Okada model 
Prior to Clawpack 5.2.1, the fault parameters we specified in a .cfg file,
but now they are explicit below.
    
Call functions with makeplots==True to create plots of topo, slip, and dtopo.
"""

from __future__ import absolute_import
from __future__ import print_function
import os

import clawpack.clawutil.data

try:
    CLAW = os.environ['CLAW']
except:
    raise Exception("*** Must first set CLAW enviornment variable")

# Scratch directory for storing topo and dtopo files:
scratch_dir = os.path.join(CLAW, 'geoclaw', 'scratch')

def get_topo(makeplots=False):
    """
    Retrieve the topo file from the GeoClaw repository.
    """
    from clawpack.geoclaw import topotools
    topo_fname = 'etopo10min120W60W60S0S.asc'
    url = 'http://depts.washington.edu/clawpack/geoclaw/topo/etopo/' + topo_fname
    clawpack.clawutil.data.get_remote_file(url, output_dir=scratch_dir, 
            file_name=topo_fname, verbose=True)

    if makeplots:
        from matplotlib import pyplot as plt
        topo = topotools.Topography(os.path.join(scratch_dir,topo_fname), topo_type=2)
        topo.plot()
        fname = os.path.splitext(topo_fname)[0] + '.png'
        plt.savefig(fname)
        print("Created ",fname)


    
def make_dtopo(makeplots=False):
    """
    Create dtopo data file for deformation of sea floor due to earthquake.
    Uses the Okada model with fault parameters and mesh specified below.
    """
    from clawpack.geoclaw import dtopotools
    import numpy

    dtopo_fname = os.path.join(scratch_dir, "dtopo_usgs100227.tt3")

    # Specify subfault parameters for this simple fault model consisting
    # of a single subfault:

    usgs_subfault = dtopotools.SubFault()
    usgs_subfault.strike = 16.
    usgs_subfault.length = 450.e3
    usgs_subfault.width = 100.e3
    usgs_subfault.depth = 35.e3
    usgs_subfault.slip = 15.
    usgs_subfault.rake = 104.
    usgs_subfault.dip = 14.
    usgs_subfault.longitude = -72.668
    usgs_subfault.latitude = -35.826
    usgs_subfault.coordinate_specification = "top center"

    fault = dtopotools.Fault()
    fault.subfaults = [usgs_subfault]

    print("Mw = ",fault.Mw())

    if os.path.exists(dtopo_fname):
        print("*** Not regenerating dtopo file (already exists): %s" \
                    % dtopo_fname)
    else:
        print("Using Okada model to create dtopo file")

        x = numpy.linspace(-77, -67, 100)
        y = numpy.linspace(-40, -30, 100)
        times = [1.]

        fault.create_dtopography(x,y,times)
        dtopo = fault.dtopo
        dtopo.write(dtopo_fname, dtopo_type=3)


    if makeplots:
        from matplotlib import pyplot as plt
        if fault.dtopo is None:
            # read in the pre-existing file:
            print("Reading in dtopo file...")
            dtopo = dtopotools.DTopography()
            dtopo.read(dtopo_fname, dtopo_type=3)
            x = dtopo.x
            y = dtopo.y
        plt.figure(figsize=(12,7))
        ax1 = plt.subplot(121)
        ax2 = plt.subplot(122)
        fault.plot_subfaults(axes=ax1,slip_color=True)
        ax1.set_xlim(x.min(),x.max())
        ax1.set_ylim(y.min(),y.max())
        dtopo.plot_dZ_colors(1.,axes=ax2)
        fname = os.path.splitext(os.path.split(dtopo_fname)[-1])[0] + '.png'
        plt.savefig(fname)
        print("Created ",fname)


if __name__=='__main__':
    get_topo(False)
    make_dtopo(False)




"""
Module to create topo and qinit data files for this example.
"""

from __future__ import absolute_import
from clawpack.geoclaw.topotools import Topography
from numpy import *

def maketopo():
    """
    Output topography file for the entire domain
    """
    nxpoints = 201
    nypoints = 241
    xlower = 0.e0
    xupper = 100.e0
    ylower = 0.e0
    yupper = 50.e0
    outfile= "island.tt3"     

    topography = Topography(topo_func=topo)
    topography.x = linspace(xlower,xupper,nxpoints)
    topography.y = linspace(ylower,yupper,nypoints)
    topography.write(outfile, topo_type=3, Z_format="%22.15e")

def makeqinit():
    """
    Create qinit data file
    """
    nxpoints = 101
    nypoints = 101
    xlower = -50.e0
    xupper = 50.e0
    yupper = 50.e0
    ylower = -50.e0
    outfile= "qinit.xyz"     

    topography = Topography(topo_func=qinit)
    topography.x = linspace(xlower,xupper,nxpoints)
    topography.y = linspace(ylower,yupper,nypoints)
    topography.write(outfile, topo_type=1)

def topo(x,y):
    """
    Island
    """
    ze = -((x-40.)**2 + (y-35.)**2)/20.
    
    #z_island = where(ze>-10., 100.*exp(ze), 0.)
    z_island = where(ze>-10., 150.*exp(ze), 0.)
    z = -50 + z_island
    return z


def qinit(x,y):
    """
    Dam break
    """
    from numpy import where
    eta = where(x<10, 40., 0.)
    return eta

if __name__=='__main__':
    maketopo()
    makeqinit()



#!/usr/bin/env python
# encoding: utf-8
r"""
Shu-Osher problem
====================

Solve the one-dimensional compressible Euler equations:

.. math::
    \rho_t + (\rho u)_x & = 0 \\
    (\rho u)_t + (\rho u^2 + p)_x & = 0 \\
    E_t + (u (E + p) )_x & = 0.

The initial condition corresponds to the Shu-Osher problem
in which a shock wave impacts a sinusoidally-varying density field.

This example also demonstrates:

 - how to use an arbitrary Runge-Kutta method by simply providing the
   Butcher coefficients of the method.
 - How to use a total fluctuation solver in SharpClaw
 - How to use characteristic decomposition with an evec() routine in SharpClaw
"""

import numpy as np
from clawpack import riemann
from clawpack.riemann.euler_with_efix_1D_constants import density, momentum, energy, num_eqn

gamma = 1.4  # Ratio of specific heats

# Coefficients of Runge-Kutta method
a = np.array([[0., 0., 0., 0., 0., 0., 0.],
              [.3772689153313680, 0., 0., 0., 0., 0., 0.],
              [.3772689153313680, .3772689153313680, 0., 0., 0., 0., 0.],
              [.2429952205373960, .2429952205373960, .2429952205373960, 0., 0., 0., 0.],
              [.1535890676951260, .1535890676951260, .1535890676951260, .2384589328462900, 0., 0., 0.]])
b = np.array([.206734020864804, .206734020864804, .117097251841844, .181802560120140, .287632146308408])
c = np.array([0., .3772689153313680, .7545378306627360, .7289856616121880, .6992261359316680])

def setup(use_petsc=False,iplot=False,htmlplot=False,outdir='./_output',solver_type='sharpclaw',
        kernel_language='Fortran',use_char_decomp=False,tfluct_solver=True):

    if use_petsc:
        import clawpack.petclaw as pyclaw
    else:
        from clawpack import pyclaw

    if kernel_language =='Python':
        rs = riemann.euler_1D_py.euler_roe_1D
    elif kernel_language =='Fortran':
        rs = riemann.euler_with_efix_1D

    if solver_type=='sharpclaw':
        solver = pyclaw.SharpClawSolver1D(rs)
        solver.time_integrator = 'RK'
        solver.a, solver.b, solver.c = a, b, c
        solver.cfl_desired = 0.6
        solver.cfl_max = 0.7
        if use_char_decomp:
            from pyclaw.sharpclaw import euler_sharpclaw1               # Import custom Fortran code
            solver.fmod = euler_sharpclaw1
            solver.tfluct_solver = tfluct_solver     # Use total fluctuation solver for efficiency
            if solver.tfluct_solver:
                    from pyclaw.sharpclaw import euler_tfluct1
                    solver.tfluct = euler_tfluct1
            solver.lim_type = 2             # WENO reconstruction
            solver.char_decomp = 2          # characteristic-wise reconstruction
    else:
        solver = pyclaw.ClawSolver1D(rs)

    solver.kernel_language = kernel_language

    solver.bc_lower[0]=pyclaw.BC.extrap
    solver.bc_upper[0]=pyclaw.BC.extrap

    mx = 400;
    x = pyclaw.Dimension(-5.0,5.0,mx,name='x')
    domain = pyclaw.Domain([x])
    state = pyclaw.State(domain,num_eqn)

    state.problem_data['gamma']= gamma

    if kernel_language =='Python':
        state.problem_data['efix'] = False

    xc = state.grid.p_centers[0]
    epsilon = 0.2
    velocity = (xc<-4.)*2.629369
    pressure = (xc<-4.)*10.33333 + (xc>=-4.)*1.

    state.q[density ,:] = (xc<-4.)*3.857143 + (xc>=-4.)*(1+epsilon*np.sin(5*xc))
    state.q[momentum,:] = velocity * state.q[density,:]
    state.q[energy  ,:] = pressure/(gamma - 1.) + 0.5 * state.q[density,:] * velocity**2

    claw = pyclaw.Controller()
    claw.tfinal = 1.8
    claw.solution = pyclaw.Solution(state,domain)
    claw.solver = solver
    claw.num_output_times = 10
    claw.outdir = outdir
    claw.setplot = setplot
    claw.keep_copy = True

    return claw

#--------------------------
def setplot(plotdata):
#--------------------------
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of visclaw.data.ClawPlotData.
    Output: a modified version of plotdata.
    """ 
    plotdata.clearfigures()  # clear any old figures,axes,items data

    # Figure for density
    plotfigure = plotdata.new_plotfigure(name='', figno=0)

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(211)'
    plotaxes.title = 'Density'
    plotaxes.xlimits = (-5.,5.)

    plotitem = plotaxes.new_plotitem(plot_type='1d')
    plotitem.plot_var = density
    plotitem.kwargs = {'linewidth':3}
    
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.title = 'Energy'
    plotaxes.axescmd = 'subplot(212)'

    plotitem = plotaxes.new_plotitem(plot_type='1d')
    plotitem.plot_var = energy
    plotitem.kwargs = {'linewidth':3}
    plotaxes.xlimits = (-5.,5.)
    
    return plotdata

if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(setup,setplot)



#!/usr/bin/env python
# encoding: utf-8
r"""
Routines for reading and writing a petsc-style output file.

These routines preserve petclaw/pyclaw syntax for i/o while taking advantage of
PETSc's parallel i/o capabilities to allow for parallel reads and writes of
frame data.
"""

from __future__ import absolute_import
from __future__ import print_function
from petsc4py import PETSc
import pickle
import os
from six.moves import range
    

def write(solution,frame,path='./',file_prefix='claw',write_aux=False,
          options={},write_p=False):
    r"""
        Write out pickle and PETSc data files representing the
        solution.  Common data is written from process 0 in pickle
        files.  Shared data is written from all processes into PETSc
        data files.
        
    :Input:
     - *solution* - (:class:`~pyclaw.solution.Solution`) pyclaw
       object to be output
     - *frame* - (int) Frame number
     - *path* - (string) Root path
     - *file_prefix* - (string) Prefix for the file name. ``default =
        'claw'``
     - *write_aux* - (bool) Boolean controlling whether the associated 
       auxiliary array should be written out. ``default = False``     
     - *options* - (dict) Optional argument dictionary, see 
        `PETScIO Option Table`_
     
     .. _`PETScIO Option Table`:
     
     format   : one of 'ascii' or 'binary'
     clobber  : if True (Default), files will be overwritten
    """    
    # Use defaults for options not passed in 
    clobber     = options.get('clobber', True)
    file_format = options.get('format', 'binary')

    if solution.num_aux == 0:
        write_aux = False

    filenames = set_filenames(frame,path,file_format,file_prefix,write_aux)
       
    if not clobber:
        for f in filenames.values():
            if os.path.exists(f):
                raise IOError('Cowardly refusing to clobber %s!' % f)

    rank =  PETSc.Comm.getRank(PETSc.COMM_WORLD)
    if rank==0:
        # Write metadata
        with open(filenames['metadata'],'wb') as metadata_file:
            # explicitly dumping a dictionary here to help out anybody trying to read the pickle file
            sol_dict = {'t':solution.t,'num_eqn':solution.num_eqn,'nstates':len(solution.states),
                             'num_aux':solution.num_aux,'num_dim':solution.domain.num_dim,
                             'write_aux':write_aux,
                             'problem_data' : solution.problem_data,
                             'mapc2p': solution.state.grid.mapc2p,
                             'file_format':file_format}
            if write_p:
                sol_dict['num_eqn'] = solution.mp

            pickle.dump(sol_dict, metadata_file)

            for state in solution.states:
                patch = state.patch
                pickle.dump({'level':patch.level,'names':patch.name,
                             'lower':patch.lower_global,
                             'num_cells':patch.num_cells_global,
                             'delta':patch.delta}, metadata_file)


    # Now write data
    q_viewer = set_up_viewers(filenames['q'],file_format.lower(),PETSc.Viewer.Mode.WRITE)
    if write_aux:
        aux_viewer = set_up_viewers(filenames['aux'],file_format.lower(),PETSc.Viewer.Mode.WRITE)
    
    for state in solution.states:
        patch = state.patch
        # we will reenable this bad boy when we switch over to petsc-dev:
        # state.q_da.view(q_viewer)
        if write_p:
            state.gpVec.view(q_viewer)
        else:
            state.gqVec.view(q_viewer)
        
        if write_aux:
            state.gauxVec.view(aux_viewer)
    
    q_viewer.flush()
    if write_aux:
        aux_viewer.flush()
    q_viewer.destroy() # Destroys aux_viewer also


def read(solution,frame,path='./',file_prefix='claw',read_aux=False,options={}):
    r"""
    Read in pickles and PETSc data files representing the solution
    
    :Input:
     - *solution* - (:class:`~pyclaw.solution.Solution`) Solution object to 
       read the data into.
     - *frame* - (int) Frame number to be read in
     - *path* - (string) Path to the current directory of the file
     - *file_prefix* - (string) Prefix of the files to be read in.  
       ``default = 'fort'``
     - *read_aux* (bool) Whether or not an auxiliary file will try to be read 
       in.  ``default = False``
     - *options* - (dict) Optional argument dictionary, see 
       `PETScIO Option Table`_
    
    .. _`PETScIO Option Table`:
    
    format   : one of 'ascii' or 'binary'
     
    """
    # Default format is binary
    file_format = options.get('format', 'binary')

    filenames = set_filenames(frame,path,file_format,file_prefix,read_aux)

    if read_aux:
        if not os.path.exists(filenames['aux']):
            # If no aux file for this frame, assume it is time-independent
            filenames['aux'] = os.path.join(path, '%s_aux.ptc' % file_prefix) + str(0).zfill(4)

    try:
        metadata_file = open(filenames['metadata'],'rb')
    except IOError:
        print("Error: file " + filenames['metadata'] + " does not exist or is unreadable.")
        raise

    # this dictionary is mostly holding debugging information, only nstates is needed
    # most of this information is explicitly saved in the individual patches
    value_dict = pickle.load(metadata_file)
    nstates    = value_dict['nstates']                    
    num_dim       = value_dict['num_dim']
    num_aux       = value_dict['num_aux']
    num_eqn       = value_dict['num_eqn']

    if read_aux and not os.path.exists(filenames['aux']):
        # Don't abort if aux file is missing
        from warnings import warn
        aux_file_path = os.path.join(path,filenames['aux'])
        warn('read_aux=True but aux file %s does not exist' % aux_file_path)
        read_aux = False

    q_viewer = set_up_viewers(filenames['q'],file_format.lower(),PETSc.Viewer.Mode.READ)
    if read_aux:
        aux_viewer = set_up_viewers(filenames['aux'],file_format.lower(),PETSc.Viewer.Mode.READ)

    patches = []
    for m in range(nstates):
        patch_dict = pickle.load(metadata_file)

        level   = patch_dict['level']
        names   = patch_dict['names']
        lower   = patch_dict['lower']
        n       = patch_dict['num_cells']
        d       = patch_dict['delta']

        from clawpack import petclaw
        dimensions = []
        for i in range(num_dim):
            dimensions.append(
                petclaw.Dimension(lower[i],lower[i] + n[i]*d[i],n[i],name=names[i]))
        patch = petclaw.Patch(dimensions)
        patch.level = level 
        state = petclaw.State(patch,num_eqn,num_aux)
        state.t = value_dict['t']
        state.problem_data = value_dict.get('problem_data',{})
        if 'mapc2p' in value_dict:
            # If no mapc2p is provided, leave the default identity map in grid
            state.grid.mapc2p = value_dict['mapc2p']

#       DA View/Load is broken in Petsc-3.1.8, we can load/view the DA if needed in petsc-3.2
#       state.q_da.load(q_viewer)
        state.gqVec.load(q_viewer)
        
        if read_aux:
            state.gauxVec.load(aux_viewer)
        
        solution.states.append(state)
        patches.append(state.patch)
    solution.domain = petclaw.geometry.Domain(patches)

    metadata_file.close()
    q_viewer.destroy() # Destroys aux_viewer also


def read_t(frame,path='./',file_prefix='claw'):
    r"""Read only the petsc.pkl file and return the data
    
    :Input:
     - *frame* - (int) Frame number to be read in
     - *path* - (string) Path to the current directory of the file
     - *file_prefix* - (string) Prefix of the files to be read in.  
       ``default = 'claw'``
     
    :Output:
     - (list) List of output variables
      - *t* - (int) Time of frame
      - *num_eqn* - (int) Number of equations in the frame
      - *npatches* - (int) Number of patches
      - *num_aux* - (int) Auxiliary value in the frame
      - *num_dim* - (int) Number of dimensions in q and aux
    
    """
    import logging
    logger = logging.getLogger('io')

    base_path = os.path.join(path,)
    path = os.path.join(base_path, '%s.pkl' % file_prefix) + str(frame).zfill(4)
    try:
        f = open(path,'rb')
    except IOError:
        print("Error: file " + path + " does not exist or is unreadable.")
        raise
    logger.debug("Opening %s file." % path)
    patch_dict = pickle.load(f)

    t      = patch_dict['t']
    num_eqn   = patch_dict['num_eqn']
    nstates = patch_dict['nstates']                    
    num_aux   = patch_dict['num_aux']                    
    num_dim   = patch_dict['num_dim']

    f.close()
        
    return t,num_eqn,nstates,num_aux,num_dim


def set_up_viewers(filename,file_format,mode):
    v = PETSc.Viewer()
    opts = {}
    if file_format == 'ascii':
        create_viewer = v.createASCII
    elif file_format == 'vtk':
        create_viewer = v.createASCII
        opts['format'] = PETSc.Viewer.Format.ASCII_VTK
    elif file_format == 'hdf5':
        create_viewer = v.createHDF5
    elif file_format == 'netcdf':
        create_viewer = v.createNetCDF
    elif file_format == 'binary':
        if hasattr(PETSc.Viewer,'createMPIIO'):
            create_viewer = v.createMPIIO
        else:
            create_viewer = v.createBinary
    else:
        raise IOError('PETSc has no viewer for the output format %s ' % file_format)

    viewer = create_viewer(filename, mode, **opts)
    return viewer


def set_filenames(frame,path,file_format,file_prefix,do_aux):
    filenames = {}
    filenames['metadata'] = os.path.join(path, '%s.pkl' % file_prefix) + str(frame).zfill(4)
    if file_format == 'vtk':
        filenames['q'] = os.path.join(path, file_prefix+str(frame).zfill(4)+'.vtk')
    else:
        filenames['q'] = os.path.join(path, '%s.ptc' % file_prefix) + str(frame).zfill(4)

    if do_aux:
        filenames['aux'] = os.path.join(path, '%s_aux.ptc' % file_prefix) + str(frame).zfill(4)

    return filenames




import os, sys
from clawpack.visclaw import geoplot
from clawpack.geoclaw.nonuniform_grid_tools import make_mapc2p
from clawpack.clawutil.data import ClawData
import numpy


xlimits = [-14,8.19]


outdir2 = None
#outdir2 = os.path.abspath('_output_ms')

if outdir2:
    if not os.path.isdir(outdir2):
        outdir2 = None

def setplot(plotdata):

    plotdata.clearfigures()

    outdir1 = plotdata.outdir
    mapc2p1, mx_edge, xp_edge = make_mapc2p(os.path.join(outdir1,'celledges.data'))
    
    from clawpack.amrclaw.data import GaugeData 
    setgauges = GaugeData() 
    setgauges.read(outdir1)
    gauge_xc = {}
    for k in range(len(setgauges.gauges)):
        gauge = setgauges.gauges[k]
        gaugeno = gauge[0]
        gauge_xc[gaugeno] = gauge[1]
    

    if outdir2:
        mapc2p2, mx_edge, xp_edge = make_mapc2p(os.path.join(outdir2,'celledges.data'))


    def fix_layout(current_data):
        from pylab import tight_layout
        tight_layout()

        
    plotfigure = plotdata.new_plotfigure(name='domain', figno=0)
    plotfigure.kwargs = {'figsize':(8,7)}
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(311)'
    plotaxes.xlimits = xlimits
    plotaxes.ylimits = [-0.05,0.2]
    plotaxes.grid = True
    plotaxes.title = 'Surface displacement'

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = geoplot.surface
    plotitem.color = 'b'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    if outdir2:
        plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
        plotitem.outdir = outdir2
        plotitem.plot_var = geoplot.surface
        plotitem.plotstyle = 'r--'
        plotitem.MappedGrid = True
        plotitem.mapc2p = mapc2p2

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    #plotitem.show = False
    plotitem.plot_var = geoplot.topo
    plotitem.color = 'g'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(312)'
    plotaxes.xlimits = xlimits
    plotaxes.ylimits = [-0.6,0.6]
    plotaxes.grid = True
    plotaxes.title = 'Velocity'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = geoplot.u_velocity
    plotitem.color = 'b'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    if outdir2:
        plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
        plotitem.outdir = outdir2
        plotitem.plot_var = geoplot.u_velocity
        plotitem.plotstyle = 'r--'
        plotitem.MappedGrid = True
        plotitem.mapc2p = mapc2p2

    plotaxes = plotfigure.new_plotaxes()
    plotaxes.axescmd = 'subplot(313)'
    plotaxes.xlimits = xlimits
    plotaxes.ylimits = [-0.25,0.1]
    plotaxes.grid = True
    plotaxes.title = 'Full depth'
    plotaxes.afteraxes = fix_layout
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1
    
    plotitem = plotaxes.new_plotitem(plot_type='1d_fill_between')
    #plotitem.show = False
    plotitem.plot_var = geoplot.surface
    plotitem.plot_var2 = geoplot.topo
    plotitem.color = [.5,.5,1]
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    #plotitem.show = False
    plotitem.plot_var = geoplot.topo
    plotitem.color = 'g'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1

    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = geoplot.surface
    plotitem.color = 'b'
    plotitem.MappedGrid = True
    plotitem.mapc2p = mapc2p1




    #-----------------------------------------
    # Figures for gauges
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='q', figno=300, \
                                         type='each_gauge')
    plotfigure.clf_each_gauge = True
    plotfigure.figsize = (12,3)

    plotaxes = plotfigure.new_plotaxes()
    
    def fixgauge(current_data):
        from pylab import title
        gaugeno = current_data.gaugeno
        xc = gauge_xc[gaugeno]
        xp = mapc2p1(xc)
        #print('+++ xc,xp:', xc,xp)
        title('Surface elevation at Gauge %i, x = %.0f m' \
              % (gaugeno, xp))

    plotaxes.afteraxes = fixgauge
    plotaxes.xlimits = [0,25]
    plotaxes.ylimits = [-0.02, 0.08]
    plotaxes.grid = True
    plotaxes.title = 'Surface elevation eta'
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 2  # eta
    plotitem.plotstyle = 'b-'


    #-----------------------------------------

    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via pyclaw.plotters.frametools.printframes:
    
    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'          # list of frames to print
    plotdata.print_gaugenos = 'all'          # list of gauges to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.html_homelink = '../README.html'   # pointer for top of index
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?
    plotdata.parallel = True                 # make multiple frame png's at once

    return plotdata



#!/usr/bin/env python
# encoding: utf-8

r"""
One-dimensional advection
=========================

Solve the linear advection equation:

.. math:: 
    q_t + u q_x = 0.

Here q is the density of some conserved quantity and u is the velocity.

The initial condition is a Gaussian and the boundary conditions are periodic.
The final solution is identical to the initial data because the wave has
crossed the domain exactly once.
"""
import numpy as np
from clawpack import riemann

def setup(nx=400, kernel_language='Python', use_petsc=False, solver_type='classic', weno_order=5, 
          time_integrator='SSP104', outdir='./_output'):

    if use_petsc:
        import clawpack.petclaw as pyclaw
    else:
        from clawpack import pyclaw

    if kernel_language == 'Fortran':
        riemann_solver = riemann.advection_1D
    elif kernel_language == 'Python':
        riemann_solver = riemann.advection_1D_py.advection_1D
            
    if solver_type=='classic':
        solver = pyclaw.ClawSolver1D(riemann_solver)
    elif solver_type=='sharpclaw':
        solver = pyclaw.SharpClawSolver1D(riemann_solver)
        solver.weno_order = weno_order
        solver.time_integrator = time_integrator
        if time_integrator == 'SSPLMMk3':
            solver.lmm_steps = 5
            solver.check_lmm_cond = True
    else: raise Exception('Unrecognized value of solver_type.')

    solver.kernel_language = kernel_language

    solver.bc_lower[0] = pyclaw.BC.periodic
    solver.bc_upper[0] = pyclaw.BC.periodic

    x = pyclaw.Dimension(0.0,1.0,nx,name='x')
    domain = pyclaw.Domain(x)
    state = pyclaw.State(domain,solver.num_eqn)

    state.problem_data['u'] = 1.  # Advection velocity

    # Initial data
    xc = state.grid.x.centers
    beta = 100; gamma = 0; x0 = 0.75
    state.q[0,:] = np.exp(-beta * (xc-x0)**2) * np.cos(gamma * (xc - x0))

    claw = pyclaw.Controller()
    claw.keep_copy = True
    claw.solution = pyclaw.Solution(state,domain)
    claw.solver = solver

    if outdir is not None:
        claw.outdir = outdir
    else:
        claw.output_format = None

    claw.tfinal =1.0
    claw.setplot = setplot

    return claw

def setplot(plotdata):
    """ 
    Plot solution using VisClaw.
    """ 
    plotdata.clearfigures()  # clear any old figures,axes,items data

    plotfigure = plotdata.new_plotfigure(name='q', figno=1)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.ylimits = [-.2,1.0]
    plotaxes.title = 'q'

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 0
    plotitem.plotstyle = '-o'
    plotitem.color = 'b'
    plotitem.kwargs = {'linewidth':2,'markersize':5}
    
    return plotdata

 
if __name__=="__main__":
    from clawpack.pyclaw.util import run_app_from_main
    output = run_app_from_main(setup,setplot)




""" 
Set up the plot figures, axes, and items to be done for each frame.

This module is imported by the plotting routines and then the
function setplot is called to set the plot parameters.
    
""" 

#--------------------------
from __future__ import absolute_import
def setplot(plotdata):
#--------------------------
    
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of visclaw.data.ClawPlotData.
    Output: a modified version of plotdata.
    
    """ 


    from visclaw import colormaps
    from matplotlib import cm

    plotdata.clearfigures()  # clear any old figures,axes,items data
    

    # Figure for pressure
    # -------------------

    plotfigure = plotdata.new_plotfigure(name='Density', figno=0)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = 'auto'
    plotaxes.title = 'Density'
    plotaxes.scaled = True      # so aspect ratio is 1
    plotaxes.afteraxes = label_axes

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_schlieren')
    #plotitem.pcolor_cmin = 0.5
    #plotitem.pcolor_cmax=3.5
    plotitem.plot_var = 0
    plotitem.add_colorbar = False
    plotitem.show = True       # show on plot?
    

    plotfigure = plotdata.new_plotfigure(name='Tracer', figno=1)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = 'auto'
    plotaxes.title = 'Tracer'
    plotaxes.scaled = True      # so aspect ratio is 1
    plotaxes.afteraxes = label_axes

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.pcolor_cmin = 0.
    plotitem.pcolor_cmax=1.0
    plotitem.plot_var = 4
    plotitem.pcolor_cmap = colormaps.yellow_red_blue
    plotitem.add_colorbar = False
    plotitem.show = True       # show on plot?
    

    plotfigure = plotdata.new_plotfigure(name='Energy', figno=2)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = 'auto'
    plotaxes.title = 'Energy'
    plotaxes.scaled = True      # so aspect ratio is 1
    plotaxes.afteraxes = label_axes

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.pcolor_cmin = 2.
    plotitem.pcolor_cmax=18.0
    plotitem.plot_var = 3
    plotitem.pcolor_cmap = colormaps.yellow_red_blue
    plotitem.add_colorbar = False
    plotitem.show = True       # show on plot?
    



    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via visclaw.frametools.printframes:

    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'          # list of frames to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.html_homelink = '../README.html'   # pointer for top of index
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?

    return plotdata

def label_axes(current_data):
    import matplotlib.pyplot as plt
    plt.xlabel('z')
    plt.ylabel('r')
    #plt.draw()
    



#!/usr/bin/env python
# encoding: utf-8
r"""
Pyclaw utility methods
"""

import time
import os
import sys
import subprocess
import logging
import tempfile
import inspect
import warnings

import numpy as np
import contextlib


LOGGING_LEVELS = {0:logging.CRITICAL,
                  1:logging.ERROR,
                  2:logging.WARNING,
                  3:logging.INFO,
                  4:logging.DEBUG}

def add_parent_doc(parent):
    """add parent documentation for a class""" 
    
    return """
    **Parent Class Documentation:**
    """ + parent.__doc__

def run_serialized(fun):
    """ Decorates a function to only run serially, even if called in parallel.

    In a parallel communicator, the first process will run while the remaining processes
    block on a barrier.  In a serial run, the function will be called directly.

    This currently assumes the global communicator is PETSc.COMM_WORLD, but is easily
    generalized.
    """

    try:
        from petsc4py import PETSc
        is_parallel = True
    except ImportError:
        is_parallel = False

    if is_parallel:
        rank = PETSc.COMM_WORLD.getRank()
        if rank == 0:
            def serial_fun(*args, **kwargs):
                fun(*args, **kwargs)
                PETSc.COMM_WORLD.Barrier()
        else:
            def serial_fun(*args, **kwargs):
                PETSc.COMM_WORLD.Barrier()
    else:
        def serial_fun(*args, **kwargs):
            fun(*args, **kwargs)

    return serial_fun

@run_serialized
def inplace_build(working_dir, warn=True):
    """Build missing extension modules with an in-place build.  This is a convenience
    function for PyClaw applications that rely on custom extension modules.  In its default
    mode, this function emits warnings indicating its actions.

    This function is safe to execute in parallel, it will only run on the 0 zero process while
    the other processes block.
    """

    if warn:
        warnings.warn("missing extension modules")
        warnings.warn("running python setup.py build_ext -i in %s" % working_dir)

    subprocess.check_call('python setup.py build_ext -i', shell=True, cwd=working_dir)

    if warn:
        warnings.warn("successfully executed python setup.py build_ext -i in %s" % working_dir)

def run_app_from_main(application,setplot=None):
    r"""
    Runs an application from pyclaw/examples/, automatically parsing command line keyword
    arguments (key=value) as parameters to the application, with positional
    arguments being passed to PETSc (if it is enabled).

    Perhaps we should take the PETSc approach of having a database of PyClaw
    options that can be queried for options on specific objects within the
    PyClaw runtime instead of front-loading everything through the application
    main...
    """

    # Arguments to the PyClaw should be keyword based, positional arguments
    # will be passed to PETSc
    petsc_args, pyclaw_kwargs = _info_from_argv(sys.argv)

    if 'use_petsc' in pyclaw_kwargs and pyclaw_kwargs['use_petsc']:
        import petsc4py
        petsc_args = [arg.replace('--','-') for arg in sys.argv[1:] if '=' not in arg]
        petsc4py.init(petsc_args)
        from clawpack import petclaw as pyclaw
    else:
        from clawpack import pyclaw

    if sys.version_info >= (2, 7):
        app_kwargs = {key: value for key, value in pyclaw_kwargs.items() 
                      if not key in ('htmlplot','iplot')}
    else:
        # the above fails with Python < 2.7, so write it out...
        app_kwargs = {}
        for key,value in pyclaw_kwargs.items():
            if key not in ('htmlplot','iplot'):
                app_kwargs[key] = value

    claw=application(**app_kwargs)

    # Solve
    status = claw.run()

    # Plot results
    htmlplot = pyclaw_kwargs.get('htmlplot',False)
    iplot    = pyclaw_kwargs.get('iplot',False)
    outdir   = pyclaw_kwargs.get('outdir','./_output')
    if htmlplot:  
        if setplot is not None:
            pyclaw.plot.html_plot(outdir=outdir,setplot=setplot)
        else:
            pyclaw.plot.html_plot(outdir=outdir)
    if iplot:     
        if setplot is not None:
            pyclaw.plot.interactive_plot(outdir=outdir,setplot=setplot)
        else:
            pyclaw.plot.interactive_plot(outdir=outdir)

    return claw

class VerifyError(Exception):
    pass

def gen_variants(application, verifier, kernel_languages=('Fortran',), disable_petsc=False, **kwargs):
    r"""
    Generator of runnable variants of a test application given a verifier

    Given an application, a script for verifying its output, and a
    list of kernel languages to try, generates all possible variants of the
    application to try by taking a product of the available kernel_languages and
    (petclaw/pyclaw).  For many applications, this will generate 4 variants:
    the product of the two main kernel languages ('Fortran' and 'Python'), against
    the the two parallel modes (petclaw and pyclaw).

    For more information on how the verifier function should be implemented,
    see util.test_app for a description, and util.check_diff for an example.

    All unrecognized keyword arguments are passed through to the application.
    """

    arg_dicts = build_variant_arg_dicts(kernel_languages, disable_petsc)

    for test_kwargs in arg_dicts:
        test_kwargs.update(kwargs)
        try:
            test_name = application.__module__
        except:
            test_name = inspect.getmodule(application)
        if 'solver_type' in test_kwargs:
            solver_info = 'solver_type={solver_type!s}, '
        else:
            solver_info = ''
        test = lambda: test_app(application, verifier, test_kwargs)
        test.description = '%s(%s)' % (test_name, str(kwargs))
        yield test
    return

def build_variant_arg_dicts(kernel_languages=('Fortran',), disable_petsc=False):
    import itertools

    # test petsc4py only if it is available
    use_petsc_opts = (False,)
    if not disable_petsc:
        try:
            import petsc4py
            use_petsc_opts=(True,False)
        except ImportError:
            pass # petsc starts disabled

    opt_names = 'use_petsc','kernel_language'
    opt_product = itertools.product(use_petsc_opts,kernel_languages)
    arg_dicts = [dict(list(zip(opt_names,argset))) for argset in opt_product]

    return arg_dicts

def test_app_variants(application, verifier, kernel_languages, **kwargs):

    arg_dicts = build_variant_arg_dicts(kernel_languages)

    for test_kwargs in arg_dicts:
        test_kwargs.update(kwargs)
        test_app(application, verifier, test_kwargs)
    return

def test_app(application, verifier, kwargs):
    r"""
    Test the output of a given application against its verifier method.

    This function performs the following two function calls::

        output = application(**kwargs)
        check_values = verifier(output)

    The verifier method should return None if the output is correct, otherwise
    it should return an indexed sequence of three items::

      0 - expected value
      1 - test value
      2 - string describing the tolerance type (abs/rel) and value.

    This information is used to present descriptive help if an error is detected.
    For an example verifier method, see util.check_diff

    """
    print(kwargs)

    if 'use_petsc' in kwargs and not kwargs['use_petsc']:
        try:
            # don't duplicate serial test runs
            from petsc4py import PETSc
            rank = PETSc.COMM_WORLD.getRank()
            if rank != 0:
                return
        except ImportError as e:
            pass

    claw = application(**kwargs)
    claw.run()
    check_values = verifier(claw)

    if check_values is not None:
        err = \
        """%s
********************************************************************************
verification function
%s
args                 : %s
norm of expected data: %s
norm of test data    : %s
test error           : %s
%s
********************************************************************************
""" % \
        (inspect.getsourcefile(application),
         inspect.getsource(verifier),
         kwargs,
         check_values[0],
         check_values[1],
         check_values[2],
         check_values[3])
        raise VerifyError(err)
    return

def check_diff(expected, test, **kwargs):
    r"""
    Checks the difference between expected and test values, return None if ok

    This function expects either the keyword argument 'abstol' or 'reltol'.
    """
    if 'delta' in kwargs:
        d = np.prod(kwargs['delta'])
    else:
        d = 1.0
    err_norm = d*np.linalg.norm(expected - test)
    expected_norm = d*np.linalg.norm(expected)
    test_norm = d*np.linalg.norm(test)
    if 'abstol' in kwargs:
        if err_norm < kwargs['abstol']: return None
        else: return (expected_norm, test_norm, err_norm,
                      'abstol  : %s' % kwargs['abstol'])
    elif 'reltol' in kwargs:
        if err_norm/expected_norm < kwargs['reltol']: return None
        else: return (expected_norm, test_norm, err_norm,
                      'reltol  : %s' % kwargs['reltol'])
    else:
        raise Exception('Incorrect use of check_diff verifier, specify tol!')

def check_solutions_are_same(sol_a,sol_b):
    assert len(sol_a.states) == len(sol_b.states)
    assert sol_a.t == sol_b.t
    for state in sol_a.states:
        for ref_state in sol_b.states:
            if ref_state.patch.patch_index == state.patch.patch_index:
                break

        # Required state attributes
        assert np.linalg.norm(state.q - ref_state.q) < 1.e-6 # Not sure why this can be so large
        if ref_state.aux is not None:
            assert np.linalg.norm(state.aux - ref_state.aux) < 1.e-16
        for attr in ['t', 'num_eqn', 'num_aux']:
            assert getattr(state,attr) == getattr(ref_state,attr)
        # Optional state attributes
        for attr in ['patch_index', 'level']:
            if hasattr(ref_state,attr):
                assert getattr(state,attr) == getattr(ref_state,attr)

        patch = state.patch
        ref_patch = ref_state.patch
        # Required patch attributes
        for attr in ['patch_index', 'level']:
            assert getattr(patch,attr) == getattr(ref_patch,attr)

        dims = patch.dimensions
        ref_dims = ref_patch.dimensions
        for dim, ref_dim in zip(dims,ref_dims):
            # Required dim attributes
            for attr in ['num_cells','lower','delta']:
                assert getattr(dim,attr) == getattr(ref_dim,attr)
            # Optional dim attributes
            for attr in ['units','on_lower_boundary','on_upper_boundary']:
                if hasattr(ref_dim,attr):
                    assert getattr(dim,attr) == getattr(ref_dim,attr)
# ============================================================================
#  F2PY Utility Functions
# ============================================================================
def compile_library(source_list,module_name,interface_functions=[],
                        local_path='./',library_path='./',f2py_flags='',
                        FC=None,FFLAGS=None,recompile=False,clean=False):
    r"""
    Compiles and wraps fortran source into a callable module in python.

    This function uses f2py to create an interface from python to the fortran
    sources in source_list.  The source_list can either be a list of names
    of source files in which case compile_library will search for the file in
    local_path and then in library_path.  If a path is given, the file will be
    checked to see if it exists, if not it will look for the file in the above
    resolution order.  If any source file is not found, an IOException is
    raised.

    The list interface_functions allows the user to specify which fortran
    functions are actually available to python.  The interface functions are
    assumed to be in the file with their name, i.e. claw1 is located in
    'claw1.f95' or 'claw1.f'.

    The interface from fortran may be different than the original function
    call in fortran so the user should make sure to check the automatically
    created doc string for the fortran module for proper use.

    Source files will not be recompiled if they have not been changed.

    One set of options of note is for enabling OpenMP, it requires the usual
    fortran flags but the OpenMP library also must be compiled in, this is
    done with the flag -lgomp.  The call to compile_library would then be:

    compile_library(src,module_name,f2py_flags='-lgomp',FFLAGS='-fopenmp')

    For complete optimization use:

    FFLAGS='-O3 -fopenmp -funroll-loops -finline-functions -fdefault-real-8'

    :Input:
     - *source_list* - (list of strings) List of source files, if these are
       just names of the source files, i.e. 'bc1.f' then they will be searched
       for in the default source resolution order, if an explicit path is
       given, i.e. './bc1.f', then the function will use that source if it can
       find it.
     - *module_name* - (string) Name of the resulting module
     - *interface_functions* - (list of strings) List of function names to
       provide access to, if empty, all functions are accessible to python.
       Defaults to [].
     - *local_path* - (string) The base path for source resolution, defaults
       to './'.
     - *library_path* - (string) The library path for source resolution,
       defaults to './'.
     - *f2py_flags* - (string) f2py flags to be passed
     - *FC* - (string) Override the environment variable FC and use it to
       compile, note that this does not replace the compiler that f2py uses,
       only the object file compilation (functions that do not have
       interfaces)
     - *FFLAGS* - (string) Override the environment variable FFLAGS and pass
       them to the fortran compiler
     - *recompile* - (bool) Force recompilation of the library, defaults to
       False
     - *clean* - (bool) Force a clean build of all source files
    """

    # Setup logger
    logger = logging.getLogger('f2py')
    temp_file = tempfile.TemporaryFile()
    logger.info('Compiling %s' % module_name)

    # Force recompile if the clean flag is set
    if clean:
        recompile = True

    # Expand local_path and library_path
    local_path = os.path.expandvars(local_path)
    local_path = os.path.expanduser(local_path)
    library_path = os.path.expandvars(library_path)
    library_path = os.path.expanduser(library_path)

    # Fetch environment variables we need for compilation
    if FC is None:
        if 'FC' in os.environ:
            FC = os.environ['FC']
        else:
            FC = 'gfortran'

    if FFLAGS is None:
        if 'FFLAGS' in os.environ:
            FFLAGS = os.environ['FFLAGS']
        else:
            FFLAGS = ''

    # Create the list of paths to sources
    path_list = []
    for source in source_list:
        # Check to see if the source looks like a path, i.e. it contains the
        # os.path.sep character
        if source.find(os.path.sep) >= 0:
            source = os.path.expandvars(source)
            source = os.path.expanduser(source)
            # This is a path, check to see if it's valid
            if os.path.exists(source):
                path_list.append(source)
                continue
            # Otherwise, take the last part of the path and try searching for
            # it in the resolution order
            source = os.path.split(source)

        # Search for the source file in local_path and then library_path
        if os.path.exists(os.path.join(local_path,source)):
            path_list.append(os.path.join(local_path,source))
            continue
        elif os.path.exists(os.path.join(library_path,source)):
            path_list.append(os.path.join(library_path,source))
            continue
        else:
            raise IOError('Could not find source file %s' % source)

    # Compile each of the source files if the object files are not present or
    # if the modification date of the source file is newer than the object
    # file's creation date
    object_list = []
    src_list = []
    for path in path_list:
        object_path = os.path.join(os.path.split(path)[0],
            '.'.join((os.path.split(path)[1].split('.')[:-1][0],'o')))

        # Check to see if this path contains one of the interface functions
        if os.path.split(path)[1].split('.')[:-1][0] in interface_functions:
            src_list.append(path)
            continue
        # If there are no interface functions specified, then all source files
        # must be included in the f2py call
        elif len(interface_functions) == 0:
            src_list.append(path)
            continue

        if os.path.exists(object_path) and not clean:
            # Check to see if the modification date of the source file is
            # greater than the object file
            if os.path.getmtime(object_path) > os.path.getmtime(path):
                object_list.append(object_path)
                continue
        # Compile the source file into the object file
        command = '%s %s -c %s -o %s' % (FC,FFLAGS,path,object_path)
        logger.debug(command)
        subprocess.call(command,shell=True,stdout=temp_file)
        object_list.append(object_path)

    # Check to see if recompile is needed
    if not recompile:
        module_path = os.path.join('.','.'.join((module_name,'so')))
        if os.path.exists(module_path):
            for src in src_list:
                if os.path.getmtime(module_path) < os.path.getmtime(src):
                    recompile = True
                    break
            for obj in object_list:
                if os.path.getmtime(module_path) < os.path.getmtime(obj):
                    recompile = True
                    break
        else:
            recompile = True

    if recompile:
        # Wrap the object files into a python module
        f2py_command = "f2py -c"
        # Add standard compiler flags
        f2py_command = ' '.join((f2py_command,f2py_flags))
        f2py_command = ' '.join((f2py_command,"--f90flags='%s'" % FFLAGS))
        # Add module names
        f2py_command = ' '.join((f2py_command,'-m %s' % module_name))
        # Add source files
        f2py_command = ' '.join((f2py_command,' '.join(src_list)))
        # Add object files
        f2py_command = ' '.join((f2py_command,' '.join(object_list)))
        # Add interface functions
        if len(interface_functions) > 0:
            f2py_command = ' '.join( (f2py_command,'only:') )
            for interface in interface_functions:
                f2py_command = ' '.join( (f2py_command,interface) )
            f2py_command = ''.join( (f2py_command,' :') )
        logger.debug(f2py_command)
        status = subprocess.call(f2py_command,shell=True,stdout=temp_file)
        if status == 0:
            logger.info("Module %s compiled" % module_name)
        else:
            logger.info("Module %s failed to compile with code %s" % (module_name,status))
            sys.exit(13)
    else:
        logger.info("Module %s is up to date." % module_name)

    temp_file.seek(0)
    logger.debug(temp_file.read())
    temp_file.close()

def construct_function_handle(path,function_name=None):
    r"""
    Constructs a function handle from the file at path.

    This function will attempt to construct a function handle from the python
    file at path.

    :Input:
     - *path* - (string) Path to the file containing the function
     - *function_name* - (string) Name of the function defined in the file
       that the handle will point to.  Defaults to the same name as the file
       without the extension.

    :Output:
     - (func) Function handle to the constructed function, None if this has
       failed.
    """
    # Determine the resulting function_name
    if function_name is None:
        function_name = path.split('/')[-1].split('.')[0]

    full_path = os.path.abspath(path)
    if os.path.exists(full_path):
        suffix = path.split('.')[-1]
        # This is a python file and we just need to read it and map it
        if suffix in ['py']:
            exec(compile(open(full_path).read(), full_path, 'exec'),globals())
            return eval('%s' % function_name)
        else:
            raise Exception("Invalid file type for function handle.")
    else:
        raise Exception("Invalid file path %s" % path)


#---------------------------------------------------------
def read_data_line(inputfile,num_entries=1,data_type=float):
#---------------------------------------------------------
    r"""
    Read data a single line from an input file

    Reads one line from an input file and returns an array of values

    inputfile: a file pointer to an open file object
    num_entries: number of entries that should be read, defaults to only 1
    type: Type of the values to be read in, they all must be the same type

    This function will return either a single value or an array of values
    depending on if num_entries > 1

    """
    l = []
    while  l==[]:  # skip over blank lines
        line = inputfile.readline()
        if line == '':
            raise IOError('*** Reached EOF in file %s' % inputfile.name)
        l = line.split()
    if num_entries == 1:  # This is a convenience for calling functions
        return data_type(l[0])
    val = np.empty(num_entries,data_type)
    if num_entries > len(l):
        print('Error in read_data_line: num_entries = ', num_entries)
        print('  is larger than length of l = ',l)
    val = [data_type(entry) for entry in l[:num_entries]]
    return val

#----------------------------------------
def convert_fort_double_to_float(number):
#----------------------------------------
    r"""
    Converts a fortran format double to a float

    Converts a fortran format double to a python float.

    number: is a string representation of the double.  Number should
    be of the form "1.0d0"

    """
    a = number.split('d')
    return float(a[0])*10**float(a[1])

#-----------------------------
def current_time(addtz=False):
#-----------------------------
    # determine current time and reformat:
    time1 = time.asctime()
    year = time1[-5:]
    day = time1[:-14]
    hour = time1[-13:-5]
    current_time = day + year + ' at ' + hour
    if addtz:
        current_time = current_time + ' ' + time.tzname[time.daylight]
    return current_time


def _method_info_from_argv(argv=None):
    """Command-line -> method call arg processing.

    - positional args:
            a b -> method('a', 'b')
    - intifying args:
            a 123 -> method('a', 123)
    - json loading args:
            a '["pi", 3.14, null]' -> method('a', ['pi', 3.14, None])
    - keyword args:
            a foo=bar -> method('a', foo='bar')
    - using more of the above
            1234 'extras=["r2"]'  -> method(1234, extras=["r2"])

    @param argv {list} Command line arg list. Defaults to `sys.argv`.
    @returns (<method-name>, <args>, <kwargs>)
    """
    import json
    if argv is None:
        argv = sys.argv

    method_name, arg_strs = argv[1], argv[2:]
    args = []
    kwargs = {}
    for s in arg_strs:
        if s.count('=') == 1:
            key, value = s.split('=', 1)
        else:
            key, value = None, s
        try:
            value = json.loads(value)
        except ValueError:
            pass
        if value=='True': value=True
        if value.lower()=='false': value=False
        if key:
            kwargs[key] = value
        else:
            args.append(value)
    return method_name, args, kwargs

def _info_from_argv(argv=None):
    """Command-line -> method call arg processing.

    - positional args:
            a b -> method('a', 'b')
    - intifying args:
            a 123 -> method('a', 123)
    - json loading args:
            a '["pi", 3.14, null]' -> method('a', ['pi', 3.14, None])
    - keyword args:
            a foo=bar -> method('a', foo='bar')
    - using more of the above
            1234 'extras=["r2"]'  -> method(1234, extras=["r2"])

    @param argv {list} Command line arg list. Defaults to `sys.argv`.
    @returns (<method-name>, <args>, <kwargs>)
    """
    import json
    if argv is None:
        argv = sys.argv

    arg_strs = argv[1:]
    args = []
    kwargs = {}
    for s in arg_strs:
        if s.count('=') == 1:
            key, value = s.split('=', 1)
        else:
            key, value = None, s
        try:
            value = json.loads(value)
        except ValueError:
            pass
        if value=='True': value=True
        if value=='False': value=False
        if key:
            kwargs[key] = value
        else:
            args.append(value)
    return args, kwargs

def _arguments_str_from_dictionary(options):
    """
    Convert method options passed as a dictionary to a str object
    having the form of the method arguments
    """
    option_string = ""
    for k in options:
        if isinstance(options[k], str):
            option_string += k+"='"+str(options[k])+"',"
        else:
            option_string += k+"="+str(options[k])+","
    option_string = option_string.strip(',')

    return option_string
#-----------------------------
class FrameCounter:
#-----------------------------
    r"""
    Simple frame counter

    Simple frame counter to keep track of current frame number.  This can
    also be used to keep multiple runs frames separated by having multiple
    counters at once.

    Initializes to 0
    """
    def __init__(self):
        self.__frame = 0

    def __repr__(self):
        return str(self.__frame)

    def increment(self):
        r"""
        Increment the counter by one
        """
        self.__frame += 1
    def set_counter(self,new_frame_num):
        r"""
        Set the counter to new_frame_num
        """
        self.__frame = new_frame_num
    def get_counter(self):
        r"""
        Get the current frame number
        """
        return self.__frame
    def reset_counter(self):
        r"""
        Reset the counter to 0
        """
        self.__frame = 0




"""
Set up the plot figures, axes, and items to be done for each frame.

This module is imported by the plotting routines and then the
function setplot is called to set the plot parameters.
"""

from __future__ import absolute_import
from clawpack.clawutil.data import ClawData
probdata = ClawData()
probdata.read('setprob.data', force=True)
ubar = probdata.u
vbar = probdata.v
vbar = probdata.w

#--------------------------
def setplot(plotdata):
#--------------------------
    
    """ 
    Specify what is to be plotted at each frame.
    Input:  plotdata, an instance of visclaw.data.ClawPlotData.
    Output: a modified version of plotdata.
    
    """ 


    from clawpack.visclaw import colormaps

    plotdata.clearfigures() # clear any old figures,axes,items data

    
    # Figure for pcolor plot
    plotfigure = plotdata.new_plotfigure(name='pcolor', figno=0)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [0,50]
    plotaxes.ylimits = [0,50]
    plotaxes.title = 'Solution'
    plotaxes.scaled = True

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = 0
    plotitem.pcolor_cmap = colormaps.red_yellow_blue
    plotitem.pcolor_cmin = -150.
    plotitem.pcolor_cmax = 150.
    plotitem.add_colorbar = True

    plotitem.amr_celledges_show = [0,0]  
    plotitem.amr_patchedges_show = [0,1]

    
    # Figure for pcolor plot
    plotfigure = plotdata.new_plotfigure(name='error pcolor', figno=10)

    def error(current_data):
        x = current_data.x
        y = current_data.y
        z = current_data.z
        t = current_data.t
        q = current_data.q
        qtrue = (x-ubar*t) + (y-vbar*t) + (z-wbar*t)
        e = q[0,:,:,:] - qtrue
        #import pdb; pdb.set_trace()
        return e

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [0,50]
    plotaxes.ylimits = [0,50]
    plotaxes.zlimits = [0,50]
    plotaxes.title = 'Error'
    plotaxes.scaled = True

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_pcolor')
    plotitem.plot_var = error
    plotitem.pcolor_cmap = colormaps.red_yellow_blue
    plotitem.pcolor_cmin = -0.000001
    plotitem.pcolor_cmax = 0.000001
    plotitem.add_colorbar = True

    plotitem.amr_celledges_show = [0,0]  
    plotitem.amr_patchedges_show = [0,1]


    # Figure for contour plot
    plotfigure = plotdata.new_plotfigure(name='contour', figno=1)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [0,50]
    plotaxes.ylimits = [0,50]
    plotaxes.zlimits = [0,50]
    plotaxes.title = 'Solution'
    plotaxes.scaled = True
    #plotaxes.afteraxes = addgauges

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_contour')
    plotitem.plot_var = 0
    plotitem.contour_nlevels = 40
    plotitem.contour_min = -150.99
    plotitem.contour_max = 150.99
    plotitem.amr_contour_colors = ['r','g','b']  # color on each level
    plotitem.amr_patch_bgcolor = ['#ffeeee', '#eeeeff', '#eeffee']
    plotitem.celledges_show = 0
    plotitem.patchedges_show = 1 


    # Figure for grid cells
    plotfigure = plotdata.new_plotfigure(name='cells', figno=2)

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = [0,50]
    plotaxes.ylimits = [0,50]
    plotaxes.title = 'Grid patches'
    plotaxes.scaled = True

    # Set up for item on these axes:
    plotitem = plotaxes.new_plotitem(plot_type='2d_patch')
    plotitem.amr_patch_bgcolor = ['#ffeeee', '#eeeeff', '#eeffee']
    plotitem.amr_celledges_show = [0,0,0]
    plotitem.amr_patchedges_show = [1,1]

    #-----------------------------------------
    # Figures for gauges
    #-----------------------------------------
    plotfigure = plotdata.new_plotfigure(name='q', figno=300, \
                    type='each_gauge')
    plotfigure.clf_each_gauge = True

    # Set up for axes in this figure:
    plotaxes = plotfigure.new_plotaxes()
    plotaxes.xlimits = 'auto'
    plotaxes.ylimits = 'auto'
    plotaxes.zlimits = 'auto'
    plotaxes.title = 'q'

    # Plot q as blue curve:
    plotitem = plotaxes.new_plotitem(plot_type='1d_plot')
    plotitem.plot_var = 0
    plotitem.plotstyle = 'b-'

    
    # Parameters used only when creating html and/or latex hardcopy
    # e.g., via visclaw.frametools.printframes:

    plotdata.printfigs = True                # print figures
    plotdata.print_format = 'png'            # file format
    plotdata.print_framenos = 'all'          # list of frames to print
    plotdata.print_fignos = 'all'            # list of figures to print
    plotdata.html = True                     # create html files of plots?
    plotdata.html_homelink = '../README.html'   # pointer for top of index
    plotdata.html_movie = 'JSAnimation'      # new style, or "4.x" for old style
    plotdata.latex = True                    # create latex file of plots?
    plotdata.latex_figsperline = 2           # layout of plots
    plotdata.latex_framesperline = 1         # layout of plots
    plotdata.latex_makepdf = False           # also run pdflatex?

    return plotdata

    
    
# To plot gauge locations on pcolor or contour plot, use this as
# an afteraxis function:

def addgauges(current_data):
    from clawpack.visclaw import gaugetools
    gaugetools.plot_gauge_locations(current_data.plotdata, \
         gaugenos='all', format_string='ko', add_labels=True)



"""
Compare DART buoy results to NOAA results.
"""

from __future__ import absolute_import
import pylab 
from pyclaw.plotters.data import ClawPlotData
from matplotlib import image

dartpng = image.imread('dart32412_comp-2.png')
pylab.figure(400)
pylab.clf()
pylab.imshow(dartpng)
pylab.hold(True)

# origin of plot in pixels on image:
t0 = 195.
y0 = 221.
tscale = 120.0
yscale = -446.66666666666669
plotdata = ClawPlotData()
plotdata.outdir = "_output"
gaugedata = plotdata.getgauge(32412)
t = gaugedata.t
eta = gaugedata.q[:,3]
t = t0 + tscale * (t / 3600.)
y = y0 + yscale*eta

pylab.plot(t,y,'b')
pylab.plot([160,225],[550,550],'b')
pylab.text(240,560,'GeoClaw (added to NOAA original)',fontsize=8)
pylab.text(100,600,'NOAA Original from http://nctr.pmel.noaa.gov/chile20100227/dart32412_comp-2.pdf',fontsize=8)


pylab.xlim([50,1300])
pylab.ylim([640,0])
pylab.axis('off')

pylab.savefig('dart.png')



r"""
Module defining Pyclaw geometry objects.
"""

import numpy as np

import warnings
deprec_message = "'edges' has been deprecated; please use 'nodes' instead."
# ============================================================================
#  Default function definitions
# ============================================================================

# Default mapc2p functions
def identity_map_1d(x):
    return x,

def identity_map_2d(x,y):
    return x,y

def identity_map_3d(x,y,z):
    return x,y,z

identity_map={'1': identity_map_1d,
              '2': identity_map_2d,
              '3': identity_map_3d}

class Grid(object):
    r"""
    Representation of a single grid.

    :Dimension information:

        Each dimension has an associated name with it that can be accessed via
        that name such as ``grid.x.num_cells`` which would access the x dimension's
        number of cells.

    :Properties:

        If the requested property has multiple values, a list will be returned
        with the corresponding property belonging to the dimensions in order.

    :Initialization:

        Input:
         - *dimensions* - (list of :class:`Dimension`) Dimensions that are to
           be associated with this grid

        Output:
         - (:class:`grid`) Initialized grid object

    A PyClaw grid is usually constructed from a tuple of PyClaw Dimension objects:

        >>> from clawpack.pyclaw.geometry import Dimension, Grid
        >>> x = Dimension(0.,1.,10,name='x')
        >>> y = Dimension(-1.,1.,25,name='y')
        >>> grid = Grid((x,y))
        >>> print(grid)
        2-dimensional domain (x,y)
        No mapping
        Extent:  [0.0, 1.0] x [-1.0, 1.0]
        Cells:  10 x 25

    We can query various properties of the grid:

        >>> grid.num_dim
        2
        >>> grid.num_cells
        [10, 25]
        >>> grid.lower
        [0.0, -1.0]
        >>> grid.delta # Returns [dx, dy]
        [0.1, 0.08]

    A grid can be extended to higher dimensions using the add_dimension() method:

        >>> z=Dimension(-2.0,2.0,21,name='z')
        >>> grid.add_dimension(z)
        >>> grid.num_dim
        3
        >>> grid.num_cells
        [10, 25, 21]

    **Coordinates:**

    We can get the x, y, and z-coordinate arrays of cell nodes and centers from the grid.
    Properties beginning with 'c' refer to the computational (unmapped) domain, while
    properties beginning with 'p' refer to the physical (mapped) domain.  For grids with
    no mapping, the two are identical.  Also note the difference between 'center' and
    'centers'.

        >>> import numpy as np
        >>> np.set_printoptions(precision=2)  # avoid doctest issues with roundoff
        >>> grid.c_center([1,2,3])
        array([ 0.15, -0.8 , -1.33])
        >>> grid.p_nodes[0][0,0,0]
        0.0
        >>> grid.p_nodes[1][0,0,0]
        -1.0
        >>> grid.p_nodes[2][0,0,0]
        -2.0

    It's also possible to get coordinates for ghost cell arrays:

        >>> x = Dimension(0.,1.,5,name='x')
        >>> grid1d = Grid([x])
        >>> grid1d.c_centers
        [array([0.1, 0.3, 0.5, 0.7, 0.9])]
        >>> grid1d.c_centers_with_ghost(2)
        [array([-0.3, -0.1,  0.1,  0.3,  0.5,  0.7,  0.9,  1.1,  1.3])]

    **Mappings:**

    A grid mapping can be used to solve in a domain that is not rectangular,
    or to adjust the local spacing of grid cells.  For instance, we can
    use smaller cells on the left and larger cells on the right by doing:

        >>> double = lambda xarr : np.array([x**2 for x in xarr])
        >>> grid1d.mapc2p = double
        >>> grid1d.p_centers
        array([0.01, 0.09, 0.25, 0.49, 0.81])

    Note that the 'nodes' (or nodes) of the mapped grid are the mapped values
    of the computational nodes.  In general, they are not the midpoints between
    mapped centers:

        >>> grid1d.p_nodes
        array([0.  , 0.04, 0.16, 0.36, 0.64, 1.  ])
    """

    def __getattr__(self,key):
        # Provide dimension attribute lists when requested from Grid object.
        # Note that this only gets called when one requests an attribute
        # that the grid itself doesn't possess.
        if key in ['num_cells','lower','upper','delta','units','centers','nodes',
                    'on_lower_boundary','on_upper_boundary']:
            return self.get_dim_attribute(key)
        else:
            raise AttributeError("'Grid' object has no attribute '"+key+"'")

    # ========== Property Definitions ========================================
    @property
    def num_dim(self):
        r"""(int) - Number of dimensions"""
        return len(self._dimensions)
    @property
    def dimensions(self):
        r"""(list) - List of :class:`Dimension` objects defining the
                grid's extent and resolution"""
        return [getattr(self,name) for name in self._dimensions]
    @property
    def c_centers(self):
        r"""(list of ndarray(...)) - List containing the arrays locating
                  the computational locations of cell centers, see
                  :meth:`_compute_c_centers` for more info."""
        self._compute_c_centers()
        return self._c_centers
    @property
    def c_nodes(self):
        r"""(list of ndarray(...)) - List containing the arrays locating
                  the computational locations of cell nodes, see
                  :meth:`_compute_c_nodes` for more info."""
        self._compute_c_nodes()
        return self._c_nodes
    @property
    def p_centers(self):
        r"""(list of ndarray(...)) - List containing the arrays locating
                  the physical locations of cell centers, see
                  :meth:`_compute_p_centers` for more info."""
        self._compute_p_centers()
        return self._p_centers
    @property
    def p_nodes(self):
        r"""(list of ndarray(...)) - List containing the arrays locating
                  the physical locations of cell nodes, see
                  :meth:`_compute_p_nodes` for more info."""
        self._compute_p_nodes()
        return self._p_nodes
    @property
    def mapc2p(self):
        return self._mapc2p
    @mapc2p.setter
    def mapc2p(self,mapc2p):
        self._mapc2p = mapc2p
        self._clear_cached_values()


    # ========== Class Methods ===============================================
    def __init__(self,dimensions):
        r"""
        Instantiate a Grid object

        See :class:`Grid` for more info.
        """

        # ========== Attribute Definitions ===================================
        r"""(func) - Coordinate mapping function"""
        self.gauges = []
        r"""(list) - List of gauges' indices to be filled by add_gauges
        method.
        """
        self.gauge_file_names  = []
        r"""(list) - List of file names to write gauge values to"""
        self.gauge_files = []
        r"""(list) - List of file objects to write gauge values to"""
        self.gauge_dir_name = '_gauges'
        r"""(string) - Name of the output directory for gauges. If the
        `Controller` class is used to run the application, this directory by
        default will be created under the `Controller` `outdir` directory.
        """
        self._p_centers = None
        self._p_nodes = None
        self._c_centers = None
        self._c_nodes = None

        # Dimension parsing
        if isinstance(dimensions,Dimension):
            dimensions = [dimensions]
        self._dimensions = []
        for dim in dimensions:
            self.add_dimension(dim)

        super(Grid,self).__init__()

    def _clear_cached_values(self):
        self._p_centers = None
        self._p_nodes = None
        self._c_centers = None
        self._c_nodes = None

    # ========== Dimension Manipulation ======================================
    def add_dimension(self,dimension):
        r"""
        Add the specified dimension to this patch

        :Input:
         - *dimension* - (:class:`Dimension`) Dimension to be added
        """

        # Add dimension to name list and as an attribute
        if dimension.name in self._dimensions:
            raise Exception('Unable to add dimension. A dimension'+
                            ' of the same name: {name}, already exists.'
                            .format(name=dimension.name))

        self._dimensions.append(dimension.name)
        setattr(self,dimension.name,dimension)
        self._clear_cached_values()
        # Reset mapping as it presumably makes no sense now
        self.mapc2p = identity_map[str(self.num_dim)]

    def get_dim_attribute(self,attr):
        r"""
        Returns a tuple of all dimensions' attribute attr
        """
        return [getattr(dim,attr) for dim in self.dimensions]

    def __copy__(self):
        return self.__class__(self)

    def __str__(self):
        output = "%s-dimensional domain " % str(self.num_dim)
        output += "("+",".join([dim.name for dim in self.dimensions])+")\n"
        if self.mapc2p in list(identity_map.values()):
            output += "No mapping\n"
            output += "Extent:  "
        else:
            output += "Mapping function: "+self.mapc2p.__name__+"\n"
            output += "Computational domain: "
        output += " x ".join(["[{:.2}, {:.2}]".format(dim.lower, dim.upper)
                             for dim in self.dimensions])
        output += "\n"
        output += "Cells:  "
        output += " x ".join(["{}".format(dim.num_cells) for dim in self.dimensions])
        return output

    # ========== Coordinates =============================================
    def _compute_c_centers(self, recompute=False):
        r"""Calculate the coordinates of the centers in the computational domain.

        :Input:
         - *recompute* - (bool) Whether to force a recompute of the arrays
        """
        if recompute or (self._c_centers is None) or \
           any([c is None for c in self.get_dim_attribute('_centers')]):
            index = np.indices(self.num_cells)
            self._c_centers = []
            for i,center_array in enumerate(self.get_dim_attribute('centers')):
                self._c_centers.append(center_array[index[i,...]])

    def _compute_c_nodes(self, recompute=False):
        r"""Calculate the coordinates of the nodes in the computational domain.

        :Input:
         - *recompute* - (bool) Whether to force a recompute of the arrays
        """
        if recompute or (self._c_nodes is None) or \
           any([c is None for c in self.get_dim_attribute('_nodes')]):
            index = np.indices(n+1 for n in self.num_cells)
            self._c_nodes = []
            for i,edge_array in enumerate(self.get_dim_attribute('nodes')):
                self._c_nodes.append(edge_array[index[i,...]])

    def _compute_p_centers(self, recompute=False):
        r"""Calculate the coordinates of the centers in the physical domain.

        :Input:
         - *recompute* - (bool) Whether to force a recompute of the arrays
        """
        if recompute or (self._p_centers is None) or \
           any([c is None for c in self.get_dim_attribute('_centers')]):
            self._compute_c_centers(recompute=recompute)
            self._p_centers = self.mapc2p(*self._c_centers)

    def _compute_p_nodes(self, recompute=False):
        r"""Calculate the coordinates of the nodes (corners) in the physical domain.

        :Input:
         - *recompute* - (bool) Whether to force a recompute of the arrays
        """
        if recompute or (self._p_nodes is None) or \
           any([c is None for c in self.get_dim_attribute('_nodes')]):
            self._compute_c_nodes(recompute=recompute)
            self._p_nodes = self.mapc2p(*self._c_nodes)

    def c_center(self,ind):
        r"""Compute center of computational cell with index ind."""

        index = [np.array(i) for i in ind]
        return np.array([self.c_centers[i][index] for i in range(self.num_dim)])

    def p_center(self,ind):
        r"""Compute center of physical cell with index ind."""
        return self.mapc2p(*self.c_center(ind))

    def c_centers_with_ghost(self, num_ghost):
        r"""
        Calculate the coordinates of the cell centers, including
        ghost cells, in the computational domain.

        :Input:
         - *num_ghost* - (int) Number of ghost cell layers
        """
        index = np.indices(n+2*num_ghost for n in self.num_cells)
        centers = []
        for i,dim in enumerate(self.dimensions):
            center_array = dim.centers_with_ghost(num_ghost)
            centers.append(center_array[index[i,...]])
        return centers

    def c_nodes_with_ghost(self, num_ghost):
        r"""
        Calculate the coordinates of the cell nodes (corners), including
        ghost cells, in the computational domain.

        :Input:
         - *num_ghost* - (int) Number of ghost cell layers
        """
        index = np.indices(n+2*num_ghost+1 for n in self.num_cells)
        nodes = []
        for i,dim in enumerate(self.dimensions):
            edge_array = dim.nodes_with_ghost(num_ghost)
            nodes.append(edge_array[index[i,...]])
        return nodes

    def p_centers_with_ghost(self,num_ghost):
        return self.mapc2p(*self.c_centers_with_ghost(num_ghost))

    def p_nodes_with_ghost(self,num_ghost):
        return self.mapc2p(*self.c_nodes_with_ghost(num_ghost))

    # ========================================================================
    # Edges: deprecated; will be removed in 6.0
    @property
    def c_edges(self):
        warnings.warn(deprec_message)
        return self.c_nodes
    @property
    def p_edges(self):
        warnings.warn(deprec_message)
        return self.p_nodes
    def p_edges_with_ghost(self,num_ghost):
        warnings.warn(deprec_message)
        return self.p_nodes_with_ghost(num_ghost)
    def c_edges_with_ghost(self, num_ghost):
        warnings.warn(deprec_message)
        return self.c_nodes_with_ghost(num_ghost)
    # ========================================================================

    # ========================================================================
    #  Gauges
    # ========================================================================
    def add_gauges(self,gauge_coords):
        r"""
        Determine the cell indices of each gauge and make a list of all gauges
        with their cell indices.
        """
        for gauge in gauge_coords:
            # Check if gauge belongs to this grid:
            if all(self.lower[n]<=gauge[n]<self.upper[n] for n in range(self.num_dim)):
                # Set indices relative to this grid
                gauge_index = [int(round((gauge[n]-self.lower[n])/self.delta[n]))
                               for n in range(self.num_dim)]
                gauge_file_name = 'gauge'+'_'.join(str(coord) for coord in gauge)+'.txt'
                self.gauge_file_names.append(gauge_file_name)
                self.gauges.append(gauge_index)

    def setup_gauge_files(self,outdir):
        r"""
        Creates and opens file objects for gauges.
        """
        import os
        gauge_path = os.path.join(outdir,self.gauge_dir_name)
        if not os.path.exists(gauge_path):
            try:
                os.makedirs(gauge_path)
            except OSError:
                print("gauge directory already exists, ignoring")

        for gauge in self.gauge_file_names:
            gauge_file = os.path.join(gauge_path,gauge)
            if os.path.isfile(gauge_file):
                os.remove(gauge_file)
            self.gauge_files.append(open(gauge_file,'a'))

    def plot(self,num_ghost=0,mapped=True,mark_nodes=False,mark_centers=False):
        r"""Make a plot of the grid.

        By default the plot uses the mapping
        grid.mapc2p and does not show any ghost cells.  This can be modified
        via the arguments `mapped` and `num_ghost`.

        Returns a handle to the plot axis object.
        """
        import matplotlib.pyplot as plt
        if self.num_dim == 2:
            fig, ax = plt.subplots(1,1)
            if num_ghost>0:
                if mapped:
                    xe, ye = self.p_nodes_with_ghost(num_ghost)
                else:
                    xe, ye = self.c_nodes_with_ghost(num_ghost)
                p = ax.pcolormesh(xe,ye,0*xe,edgecolors='k',cmap='bwr',alpha=0.2)
                p.set_clim(-1,1)
            if mapped:
                xe, ye = self.p_nodes
                xc, yc = self.p_centers
            else:
                xe, ye = self.c_nodes
                xc, yc = self.c_centers
            p = ax.pcolormesh(xe,ye,0*xe,edgecolors='k',cmap='bwr')
            p.set_clim(-1,1)
            if mark_nodes:
                ax.plot(xe,ye,'or')
            if mark_centers:
                ax.plot(xc,yc,'ob')
            ax.axis('equal')
            ax.set_xlabel(self.dimensions[0].name)
            ax.set_ylabel(self.dimensions[1].name)
            return ax
        else:
            raise Exception('Grid plotting implemented for 2D grids only.')

    def _check_validity(self):
        for dim in self.dimensions:
            dim._check_validity()
        assert type(self.num_cells) is int, 'Dimension.num_cells must be an integer'
        assert type(self.lower) is float, 'Dimension.lower must be a float'
        assert type(self.upper) is float, 'Dimension.upper must be a float'
        assert self.num_cells>0, 'Dimension.num_cells must be positive'
        assert self.upper > self.lower, 'Dimension.upper must be greater than lower'


# ============================================================================
#  Dimension Object
# ============================================================================
class Dimension(object):
    r"""
    Basic class representing a dimension of a Patch object

    :Initialization:

    Required arguments, in order:
     - *lower* - (float) Lower extent of dimension
     - *upper* - (float) Upper extent of dimension
     - *num_cells* - (int) Number of cells

    Optional (keyword) arguments:
     - *name* - (string) string Name of dimension
     - *units* - (string) Type of units, used for informational purposes only

    Output:
     - (:class:`Dimension`) - Initialized Dimension object

    Example:

    >>> from clawpack.pyclaw.geometry import Dimension
    >>> x = Dimension(0.,1.,100,name='x')
    >>> print(x)
    Dimension x:  (num_cells,delta,[lower,upper]) = (100,0.01,[0.0,1.0])
    >>> x.name
    'x'
    >>> x.num_cells
    100
    >>> x.delta
    0.01
    >>> x.nodes[0]
    0.0
    >>> x.nodes[1]
    0.01
    >>> x.nodes[-1]
    1.0
    >>> x.centers[-1]
    0.995
    >>> len(x.centers)
    100
    >>> len(x.nodes)
    101
    """

    @property
    def delta(self):
        r"""(float) - Size of an individual, computational cell"""
        return (self.upper-self.lower) / float(self.num_cells)

    # ========== Edges: deprecated; will be removed in 6.0 =======
    @property
    def edges(self):
        warnings.warn(deprec_message)
        return self.nodes

    def edges_with_ghost(self,num_ghost):
        warnings.warn(deprec_message)
        return self.nodes_with_ghost(num_ghost)
    # ========================================================================

    # ========== Centers and nodes ========================================
    @property
    def nodes(self):
        r"""(ndarrary(:)) - Location of all cell edge coordinates
        for this dimension"""
        if self._nodes is None:
            self._nodes = np.empty(self.num_cells+1)
            for i in range(0,self.num_cells+1):
                self._nodes[i] = self.lower + i*self.delta
        return self._nodes
    @property
    def centers(self):
        r"""(ndarrary(:)) - Location of all cell center coordinates
        for this dimension"""
        if self._centers is None:
            self._centers = np.empty(self.num_cells)
            for i in range(0,self.num_cells):
                self._centers[i] = self.lower + (i+0.5)*self.delta
        return self._centers
    @property
    def lower(self):
        return self._lower
    @lower.setter
    def lower(self,lower):
        self._lower = float(lower)
        self._centers = None  # Reset cached arrays
        self._nodes = None
        self._check_validity()
    @property
    def upper(self):
        return self._upper
    @upper.setter
    def upper(self,upper):
        self._upper = float(upper)
        self._centers = None  # Reset cached arrays
        self._nodes = None
        self._check_validity()
    @property
    def num_cells(self):
        return self._num_cells
    @num_cells.setter
    def num_cells(self,num_cells):
        self._num_cells = int(num_cells)
        self._centers = None  # Reset cached arrays
        self._nodes = None
        self._check_validity()

    def centers_with_ghost(self,num_ghost):
        r"""(ndarrary(:)) - Location of all cell center coordinates
        for this dimension, including centers of ghost cells."""
        centers = self.centers
        pre = self.lower+(np.arange(-num_ghost,0)+0.5)*self.delta
        post = self.upper + self.delta * (np.arange(num_ghost) + 0.5)
        return np.hstack((pre,centers,post))

    def nodes_with_ghost(self,num_ghost):
        r"""(ndarrary(:)) - Location of all edge coordinates
        for this dimension, including nodes of ghost cells."""
        nodes   = self.nodes
        pre  = np.linspace(self.lower-num_ghost*self.delta,self.lower-self.delta,num_ghost)
        post = np.linspace(self.upper+self.delta, self.upper+num_ghost*self.delta,num_ghost)
        return np.hstack((pre,nodes,post))

    def __init__(self, lower, upper, num_cells, name='x',
                 on_lower_boundary=None,on_upper_boundary=None, units=None):
        r"""
        Create a Dimension object.

        See :class:`Dimension` for full documentation
        """
        if isinstance(lower,str):
            raise Exception('Passing dimension name as first argument is deprecated. \
                             Pass it as a keyword argument instead.')

        self._nodes = None
        self._centers = None
        self._centers_with_ghost = None
        self._nodes_with_ghost = None

        self._lower = float(lower)
        self._upper = float(upper)
        self._num_cells = int(num_cells)
        self.name = name
        self.on_lower_boundary = on_lower_boundary
        self.on_upper_boundary = on_upper_boundary
        self.units = units

        self._check_validity()

    def _check_validity(self):
        assert isinstance(self.num_cells,int), 'Dimension.num_cells must be an integer; got %s' % type(self.num_cells)
        assert isinstance(self.lower,float), 'Dimension.lower must be a float'
        assert isinstance(self.upper,float), 'Dimension.upper must be a float'
        assert self.num_cells>0, 'Dimension.num_cells must be positive'
        assert self.upper > self.lower, 'Dimension.upper must be greater than lower'

    def __str__(self):
        output = "Dimension %s" % self.name
        if self.units:
            output += " (%s)" % self.units
        output += ":  (num_cells,delta,[lower,upper]) = (%s,%s,[%s,%s])" \
            % (self.num_cells,self.delta,self.lower,self.upper)
        return output

    def __len__(self):
        return self.num_cells


# ============================================================================
#  Pyclaw Patch object definition
# ============================================================================
class Patch(object):
    """
    :Global Patch information:

        Each patch has a value for :attr:`level` and :attr:`patch_index`.
    """
    # Global properties
    @property
    def num_cells_global(self):
        r"""(list) - List of the number of cells in each dimension"""
        return self.get_dim_attribute('num_cells')
    @property
    def lower_global(self):
        r"""(list) - Lower coordinate extents of each dimension"""
        return self.get_dim_attribute('lower')
    @property
    def upper_global(self):
        r"""(list) - Upper coordinate extends of each dimension"""
        return self.get_dim_attribute('upper')
    @property
    def num_dim(self):
        r"""(int) - Number of dimensions"""
        return len(self._dimensions)
    @property
    def dimensions(self):
        r"""(list) - List of :class:`Dimension` objects defining the
                grid's extent and resolution"""
        return [getattr(self,name) for name in self._dimensions]
    @property
    def delta(self):
        r"""(list) - List of computational cell widths"""
        return self.get_dim_attribute('delta')
    @property
    def name(self):
        r"""(list) - List of names of each dimension"""
        return self._dimensions

    def __init__(self,dimensions):
        self.level = 1
        r"""(int) - AMR level this patch belongs to, ``default = 1``"""
        self.patch_index = 1
        r"""(int) - Patch number of current patch, ``default = 0``"""

        if isinstance(dimensions,Dimension):
            dimensions = [dimensions]
        self._dimensions = []
        for dim in dimensions:
            dim.on_lower_boundary = True
            dim.on_upper_boundary = True
            self.add_dimension(dim)

        self.grid = Grid(dimensions)

        super(Patch,self).__init__()

    def add_dimension(self,dimension):
        r"""
        Add the specified dimension to this patch

        :Input:
         - *dimension* - (:class:`Dimension`) Dimension to be added
        """

        # Add dimension to name list and as an attribute
        if dimension.name in self._dimensions:
            raise Exception('Unable to add dimension. A dimension'+
                            ' of the same name: {name}, already exists.'
                            .format(name=dimension.name))

        self._dimensions.append(dimension.name)
        setattr(self,dimension.name,dimension)

    def get_dim_attribute(self,attr):
        r"""
        Returns a tuple of all dimensions' attribute attr
        """
        return [getattr(getattr(self,name),attr) for name in self._dimensions]

    def __deepcopy__(self,memo={}):
        import copy
        result = self.__class__(copy.deepcopy(self.dimensions))
        result.__init__(copy.deepcopy(self.dimensions))
        result.grid.mapc2p = self.grid.mapc2p

        for attr in ('level','patch_index'):
            setattr(result,attr,copy.deepcopy(getattr(self,attr)))

        return result

    def __str__(self):
        output = "Patch %s:\n" % self.patch_index
        output += '\n'.join((str(getattr(self,dim)) for dim in self._dimensions))
        return output

# ============================================================================
#  Pyclaw Domain object definition
# ============================================================================
class Domain(object):
    r"""
    A Domain is a list of Patches.

    A Domain may be initialized in the following ways:

        1. Using 3 arguments, which are in order
            - A list of the lower boundaries in each dimension
            - A list of the upper boundaries in each dimension
            - A list of the number of cells to be used in each dimension

        2. Using a single argument, which is
            - A list of dimensions; or
            - A list of patches.

    :Examples:

        >>> from clawpack import pyclaw
        >>> domain = pyclaw.Domain( (0.,0.), (1.,1.), (100,100))
        >>> print(domain.num_dim)
        2
        >>> print(domain.grid.num_cells)
        [100, 100]
    """
    @property
    def num_dim(self):
        r"""(int) - :attr:`Patch.num_dim` of base patch"""
        return self._get_base_patch_attribute('num_dim')
    @property
    def patch(self):
        r"""(:class:`Patch`) - First patch is returned"""
        return self.patches[0]
    @property
    def grid(self):
        r"""(list) - :attr:`Patch.grid` of base patch"""
        return self._get_base_patch_attribute('grid')

    def __init__(self,*arg):
        if len(arg)>1:
            lower = arg[0]
            upper = arg[1]
            n     = arg[2]
            dims = []
            names = ['x','y','z']
            names = names[:len(n)+1]
            for low,up,nn,name in zip(lower,upper,n,names):
                dims.append(Dimension(low,up,nn,name=name))
            self.patches = [Patch(dims)]
        else:
            geom = arg[0]
            if not isinstance(geom,list) and not isinstance(geom,tuple):
                geom = [geom]
            if isinstance(geom[0],Patch):
                self.patches = geom
            elif isinstance(geom[0],Dimension):
                self.patches = [Patch(geom)]

    def _get_base_patch_attribute(self, name):
        r"""
        Return base patch attribute name

        :Output:
         - (id) - Value of attribute from ``self.patches[0]``
        """
        return getattr(self.patches[0],name)

    def __deepcopy__(self,memo={}):
        import copy
        result = self.__class__(copy.deepcopy(self.patches))
        result.__init__(copy.deepcopy(self.patches))

        return result


if __name__ == "__main__":
    import doctest
    doctest.testmod()



#!/usr/bin/env python
# encoding: utf-8
"""
limiters

The PetClaw limiters all inherit from PyClaw limiters.
"""

# This __init__ script only imports common utilities, most of the import 
# should be done depending on the solver needed

from __future__ import absolute_import
from clawpack.riemann import *

__all__ = ['tvd']
from clawpack.pyclaw.limiters import tvd




#!/usr/bin/env python
# encoding: utf-8

"""I/O support for ForestClaw"""

from __future__ import absolute_import
import logging
logger = logging.getLogger('pyclaw.fileio')

from . import ascii as forestascii
from clawpack.pyclaw.fileio import ascii

ascii.read_patch_header = forestascii.read_patch_header
ascii.write_patch_header = forestascii.write_patch_header

__all__ = ["ascii.read", "ascii.write"]
